;+
; NAME:
;	ET2UTC
;
; PURPOSE:
;	Convert from ephemeris time in seconds (TDB) to UTC string format. Only use this routine
;   with MIMI data, it may not be accurate enough for other instruments!!!!!!
;
;   WARNING: This routine assumes that TDT equals TDB since the MIMI data data collection rate
;   is not affected by differences in time of less than 0.1 second. I found that this routine calculates 
;   an ET with a maximum difference of 0.00038892 seconds over one year calculated every minute, 
;   when compared with a routine using spice. To replace this routine with
;   SPICE software which requires the MIMI team SPICE analysis kernel set see the included routine 
;   SPICE_ET2UTC.pro
;
;   See "http://naif.jpl.nasa.gov/pub/naif/toolkit_docs/FORTRAN/index.html" Required Reading- Time
;   for more information.
;   
;
; CATEGORY:
;	Time Support Routine. 
;
; CALLING SEQUENCE:
;	s=ET2UTC(et, utc)
;
; INPUTS:
;	et           input array of double  ephermeris time Barycentric dynamical time (TDB) to be converted
;
; OUTPUTS:
;  utc  		output string  UTC starttime, yyyy-doyThh:mm.sss (Coordinated Universal Time)
;
;   RETURNS 0 return will always be 0, left in to make it easier to replace
;		with spice routines.
;
; REQUIRES:
;   Uses the leap second common block, assumes setup was run before use
;
;   The leap second file is obtained from "ftp://naif.jpl.nasa.gov/pub/naif/CASSINI/kernels/lsk/" 
;   and the user should check in January and August of each year to see if any leap seconds have 
;   been added.
;           
; KEYWORD PARAMETERS:
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       01 Oct 2010, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TESTET2UTC

	utcs = ['1999-001T12:00:00',$
	'2000-001T11:58:55.444',$
	'2000-001T00:00:00',$
	'2005-365T23:59:59.999',$
	'2006-001T00:00:00.000',$
	'2008-366T23:59:59.999',$
	'2009-001T00:00:00.000',$
	'2011-200T12:02:33.555',$
	'2018-200T12:20:15.234']
    diff = dblarr(n_elements(utcs))
	FOR n=0,n_elements(utcs)-1 DO BEGIN
		s = UTC2ET(utcs[n], secs)
		s = ET2UTC(secs[0], utc2)
        s=SPICE_UTC2ET(utcs[n],et)
        diff[n] = secs[0]-et[0]
		print,utcs[n],utc2,secs,et,diff[n],format='(2(A23),3(f23.8))'
		print,' '
	ENDFOR
    print,'difference '
    print,min(diff),max(diff),format='(2(f23.8))'
	
RETURN
END

FUNCTION ET2UTC, et, utc

COMMON LEAP, ls

    ;check that  leap second kernel has been read in and stored in common block
    IF  SIZE(ls,/TYPE) NE 8  THEN BEGIN
        SETUP
        IF  SIZE(ls,/TYPE) NE 8  THEN BEGIN
            PRINT,'READ_LSK: Error, please set LEAPKERNEL_FILE envirionmental variable in setup.pro and run setup'
            RETURN,-1
        ENDIF
    ENDIF

	num = N_ELEMENTS(et)
    utc = STRARR(num)
	
	FOR n=0,num-1 DO BEGIN
        ;convert et to TAI
        tai =  et[n] - DOUBLE(ls.deltaTa) 

        ;convert TAI to UTC
        utc[n] = TAI2UTC(tai[0])

	ENDFOR


RETURN,0
END

