;+
; NAME:
;	GET_INCA_ATT
;
; PURPOSE:
;	Find pointing of INCA pixels in INCA FOV frame 'CASSINI_MIMI_INCA_LL'
;	corrected by the offsets of the transmitted images if appropriate. See
;	the routine TEST_GET_INCA_ATT for an example of how to use this routine. 
;
; NOTE: 
;	This routine requires use of the mimi analysis spice kernel set
;	and the spice DLM to be installed for IDL. See 
;	http://cassini-mimi.jhuapl.edu/data_auxiliary.html#Kernels
;	for more information on how to get the analysis kernel set and load
;	it inside IDL.
;	See http://naif.jpl.nasa.gov/naif/toolkit.html for information on
;	getting the toolkit, look at Toolkits for IDL option.
;   
; NOTE: 
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	status = GET_INCA_ATT(useet, numrows, type, phi, theta, spinmode, frame, $
;				att)
;
; INPUTS:
;	useet			Double array of Ephemeris time. It can be found in data array from 
;					PDS_READ_MIMIIMG in the data[*,DATA_ET_SECS_C]. You can use UTC2ET 
;					to convert from yyyy-doyThh:mm:ss.msc format. 
;
;	numrows			Double array of number of rows (or columns, all images are square), 
;					It can be found in the data array from PDS_READ_MIMIIMG in the 
;					data[*,NUMROWS_C].

;	type			Double array of image resolution type, It can be found in 
;					the data array from PDS_READ_MIMIIMG in the data[at,TYPE_C].

;	phi				Double array of phi offset in pixels, It can be found in 
;					the data array from PDS_READ_MIMIIMG in the data[at,PHIOFFSET_C].

;	theta			Double array of theta offset in pixels, It can be found in 
;					the data array from PDS_READ_MIMIIMG in the data[at,THETAOFFSET_C].

;	spinmode		Double array of spin stare flag (spin=1, stare=0), It can be found in 
;					the data array from PDS_READ_MIMIIMG  in the data[at,SPINSTAR_MODE_C].
;
;	frame			String name of SPICE frame. The original data is calculated in the 
;					'CASSINI_MIMI_INCA_LL' frame but can be transformed into another 
;					spice frame such as 'XINCA_BORESIGHT_CENTERED'.
;					Look in the analysis_kernels_small directory in the fk subdirectory at 
;					the mimi_v???.tf file to find the MIMI specific frames and frame 
;					descriptions. Good frames to use for INCA image display are
;					XINCA_BORESIGHT_CENTERED, XINCA_SATURN_CENTERED, XINCA_TITAN_CENTERED.
;					The XINCA_BORESIGHT_CENTERED frame is a dynamically defined frame, 
;					defined as follows. The primary axis is the CASSINI_MIMI_INCA_LL 
;					boresight X axis and is labeled X. The secondary Z axis is the Z axis 
;					of the IAU_SATURN frame. The Y-axis completes the right-handed system.
;					This frame is primarily used by the XINCA program. XINCA's skymap 
;					display is defined to display every frame with the Z axis up, 
;					X axis into the page and Y axis out to the left. So in the
;					skymap, using this frame, Z or up is the spin axis of Saturn, 
;					into the page is the boresight.
;					The XINCA_SATURN_CENTERED frame is a dynamically defined frame, 
;					defined as follows. The primary axis is the CASSINI spacecraft to 
;					Saturn vector and is labeled X. The secondary Z axis is the Z axis 
;					of the IAU_SATURN frame. The Y-axis completes the right-handed system.
;					The XINCA_TITAN_CENTERED frame is a dynamically defined frame, 
;					defined as follows. The primary axis is the CASSINI spacecraft to 
;					Titan vector and is labeled X. The secondary Z axis is the Z axis 
;					of the IAU_TITAN frame. The Y-axis completes the right-handed system.
;
; OUTPUT:
;	att				3-d array of IDL structures with the following order:
;											[nimage, x(azimuth,phi), y(elevation,theta)]
;					structure fields: 	{lat:(double degrees),$
;										lon:(double degrees),$
;										v:(double array[3] rectangular cartesian coordinates)}
;					
;
; KEYWORD PARAMETERS:
;	USEET			Array of ET ephemeris times to find pointing at.
;
;
;                       Copyright (C) 2012
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       03 August 2012, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FUNCTION GET_INCA_ATT, useet, numrows, type, phi, theta, spinmode, frame, $
			att

@pds_data_hdr_h        ;include data header column indexes
@pds_image_hdr_h     ;include inca image header column indexes

	CATCH,err
	IF err NE 0 THEN BEGIN
		PRINT,'GET_INCA_ATT: Failure to find INCA pointing '
		PRINT,'Do you have the SPICE DLM installed?'
		PRINT,'Do you have the MIMI analysis kernel set installed and loaded?'
		RETURN,-1
	ENDIF
	
	

	status = 0
	CSPICE_KTOTAL,'ALL',cnts
	IF cnts GT 0 THEN BEGIN

		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;Rough representation of XY layout in array with no offsets. 
		;	When you display this with the default IDL TV, it will 
		;	display upside down. So the image will show +60 degrees 
		;	elevation at the top and -60 degrees elevation at the bottom.
		;
		; -60 degrees 
		; att[n,*,0]
		;
		; E
		; l L
		; e a T
		; v t h  
		; a i e  Y
		; t t t
		; i	u a
		; o d
		; n e
		;
		; 60 degrees 
		; att[n,*,numrow-1]
		;
		;   			 		Azimuth
		;						Longitude 
		;				 		  Phi
		;-45 degrees               X           	45 degrees
		; att[n,0,*] 							att[n,numrow-1,*]
		
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;set up for calculation of pixel position
		; in degrees in the transmitted 90 x 120
		;  degree image
		;set up INCA fov 
		fov = [-45,-60.0,45.0,60.0]
		az_bot = 0
		el_bot = 1
		az_top = 2
		el_top = 3
		az_limit = 90.
		el_limit = 120.

		spinvalue = 1
		nimages = N_ELEMENTS(numrows)
		maxnumrows = MAX(numrows)
		smallatt = {lat:0d,lon:0d,v:dblarr(3)}
		att = REPLICATE(smallatt,nimages,maxnumrows,maxnumrows)
		radius = 1.
		
		
		; Manual shift for image pixel [theta,phi] Units are in 64x64 pixel units, 
		; will shift the inca image over and up. This shift was determined by the 
		; instrument scientist.
		shift_pixel=[-1.5, 0.5]
		PDS_GET_PHITHETA_SHIFTDEG, shift_pixel[1], shift_pixel[0], phi_shift, theta_shift
		
		
		FOR n=0,nimages-1 DO BEGIN
    		az_bin = az_limit/FLOAT(numrows[n])
    		el_bin = el_limit/FLOAT(numrows[n])

    		;make arrays of azimuth and elevation values in degrees
    		azs = fov[az_bot]+az_bin/2.0+FINDGEN(numrows[n])*az_bin
    		els = fov[el_bot]+el_bin/2.0+FINDGEN(numrows[n])*el_bin
			

			;call routine to convert INCA image phi and theta offset from pixels 
			;to degrees
			PDS_GET_PHITHETA_DEG, numrows[n], phi[n], theta[n], az_shift, el_shift

			FOR x=0,numrows[n]-1 DO BEGIN
				FOR y=0,numrows[n]-1 DO BEGIN

					;If we are in stare mode or spinning and have high time 
					;resolution images, add offsets converted to 
					;degrees to each pixel vector of INCA image, everything is in degrees
					IF spinmode[n] EQ 0  OR $
					(spinmode[n] EQ spinvalue AND type[n] EQ IMAGETYPE_HIGHTIME) THEN BEGIN
                		a = azs[x] + az_shift - phi_shift
                		e = els[y] + el_shift - theta_shift
					;No offsets are used in this image mode
					ENDIF ELSE BEGIN
                		a = azs[x] - phi_shift
                		e = els[y] - theta_shift
					ENDELSE
					
					;transform vector into desired frame and frame is not INCA_LL
					IF STRMATCH(frame,'CASSINI_MIMI_INCA_LL') NE 1 THEN BEGIN
                		;Convert the inca image frame coordinates from latitudinal to rectangular
                		;negate longitude if frame is not INCA LL(which is left handed)
						CSPICE_LATREC, 1.0, -(a/!RADEG), e/!RADEG, v
						
						v_to = PDS_XFORM_VECTOR('CASSINI_MIMI_INCA_LL', useet[n], frame, useet[n], v, $
									status = status)
						IF status NE -1 THEN BEGIN
							att[n,x,y].v = v_to
							CSPICE_RECLAT,v_to, r, lon, lat
							att[n,x,y].lat = lat*!RADEG
							att[n,x,y].lon = lon*!RADEG
						ENDIF ELSE BEGIN
							PRINT,'GET_INCA_ATT: Failure to transform into new frame'
							RETURN,-1
						ENDELSE
						
					;Frame is INCA LL frame: USER REMEMBER this frame is left handed~
					ENDIF ELSE BEGIN
						;convert lat and lon to a vector
						CSPICE_LATREC, 1.0, a/!RADEG, e/!RADEG, v
						att[n,x,y].v = v
						att[n,x,y].lat = e
						att[n,x,y].lon = a
					
					ENDELSE	
				ENDFOR
			ENDFOR

		ENDFOR
	
	
	
	ENDIF ELSE BEGIN
		PRINT,'GET_INCA_ATT: Failure to find INCA pointing '
		PRINT,'Do you have the MIMI analysis kernel set installed and loaded?'
		RETURN,-1
	ENDELSE

RETURN,status
END
