;+
; NAME:
;	GET_INCA_PITCH
;
; PURPOSE:
;	Calculates the pitch angle for each inca image pixel 
;
; ASSUMPTIONS:
;   Assume inca instrument frame is centered at X (X perpendicular to plane). Y goes out to the left. Z
;   goes out the top. Assume fov [-45, -60, 45, 60] must be transformed into SPICE INCA frame and then
;   into the Cassini s/c frame. The mag data is already assumed to be in the s/c frame. 
;   Assume phi value is from a left positive longitude so positive values go clockwise.
;

; CATEGORY:
;	Plotting Routine. 
;
; CALLING SEQUENCE:
;	s=GET_INCA_PITCH(startet, stopet, data, type, mcharge, mspec, magtol, mimagesize, mphi, mtheta, mangle)
;
; INPUTS:
;   startet     start ephemeris time DBLARR(nimages)
;   stopet      stop ephemeris time DBLARR(nimages)
;   data        mag data in CASSINI_SC_COORD frame (default frame for PDS_READ_MAG_DATA)
;   mtype        resolution of images DBLARR(nimages)
;   mcharge      neutral or ion charge (commandded charge) (nimages)
;   mspec        species of image DBLARR(nimages)
;   magtol      tolerance for mag in degrees
;   mimagesize   size in pixels of image DBLARR(nimages)
;   mphi         phi offset of image DBLARR(nimages)
;   mtheta       theta offset of image DBLARR(nimages)
;
; OUTPUTS:
;   mangle:        float array, [nimages,azimuth,elevation] pitch angle in degrees
;
; KEYWORD PARAMETERS:
;
;
; CREATION:
;      08 August 2012, M.B. Kusterer 
;
; MODIFICATION HISTORY:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION GET_INCA_PITCH, startet, stopet, data, mtype, mcharge, mspec, magtol, $
                    mimagesize, mphi, mtheta, mincaspinstare, mangle

@pds_data_hdr_h        ;include data header column indexes
@pds_image_hdr_h     ;include inca image header column indexes

	nimages = N_ELEMENTS(startet)
	imagesize = MAX(mimagesize)
    mangle = FLTARR(nimages,imagesize,imagesize)
    angle = FLTARR(imagesize,imagesize)
	spinvalue = 1
	
	FOR k=0,N_ELEMENTS(startet)-1 DO BEGIN
		et = [startet[k], stopet[k]]
		type = mtype[k]
		charge = mcharge[k]
		spec = mspec[k]
		imagesize = mimagesize[k]
		phi = mphi[k]
		theta = mtheta[k]
		incaspinstare = mincaspinstare[k]

    	;set up et to be in middle of accumulation range for image
    	image_et = et[0] 

    	s=SPICE_ET2UTC(et, utc)
    	s=SPICE_ET2UTC(image_et, image_utc)

    	colpos = 11 ;start of mag data after mimi header data
    	timepos = WHERE(data[*,DATA_ET_SECS_C] GE et[0] AND $
                        	data[*,DATA_ET_SECS_C] LE et[1])
    	IF timepos(0) EQ -1 THEN BEGIN
        	print,'<<WARNING>> Programming Error, failure to find time range in data '
    	   RETURN,-1
    	ENDIF

    	n_mag = N_ELEMENTS(data[timepos,colpos])

    	mag_et = data[timepos,DATA_ET_SECS_C]
    	mag_sc = REFORM([data[timepos,colpos],data[timepos,colpos+1],data[timepos,colpos+2]],n_mag,3)
		
    	;
    	; translate phi and theta into number of degrees, based on image size
    	;
		apply_flag = 0
		IF incaspinstare EQ 0  OR $
					(incaspinstare EQ spinvalue AND type EQ IMAGETYPE_HIGHTIME) THEN apply_flag = 1
					
    	PDS_GET_PHITHETA_DEG, imagesize, phi, theta, az_shift, el_shift
    	az_shift = (apply_flag EQ 1) ? az_shift : 0.
    	el_shift = (apply_flag EQ 1) ? el_shift : 0.

    	;set up pretend INCA fov at 0,0 (X axis perpendicular to inca image center, 
    	fov = [-45,-60.0,45.0,60.0]
    	az_bot = 0
    	el_bot = 1
    	az_top = 2
    	el_top = 3

    	; Perform shift for phi and theta if not low time image in stare mode
    	IF (incaspinstare EQ 0) OR $
    	(incaspinstare EQ 1 AND type EQ IMAGETYPE_HIGHTIME) THEN BEGIN
        	fov[az_bot] = fov[az_bot] + az_shift
        	fov[az_top] = fov[az_top] + az_shift
        	fov[el_bot] = fov[el_bot] + el_shift
        	fov[el_top] = fov[el_top] + el_shift
    	ENDIF


    	;convert all mag vectors to J2000 (intertial)
    	CSPICE_PXFORM,'J2000', 'CASSINI_SC_COORD', et[0], j2000_sc_mat  ;can convert back to j2000 at time of start of 
                                                                    	;integration

    	mag_j2000 = mag_sc
    	v_temp = DBLARR(3)
    	FOR n=0,n_mag-1 DO BEGIN
        	;must convert mag data at the time of the individual vector
        	CSPICE_PXFORM,'CASSINI_SC_COORD','J2000', mag_et[n], sc_j2000_mat
        	CSPICE_MXV, sc_j2000_mat, REFORM(mag_sc[n, *]), vtemp
        	mag_j2000[n, 0] = vtemp[0]
        	mag_j2000[n, 1] = vtemp[1]
        	mag_j2000[n, 2] = vtemp[2]
    	ENDFOR

    	;compute mean in intertial coordinate system
    	bx_mean = MEAN(mag_j2000[*,0],/NAN,/DOUBLE)
    	by_mean = MEAN(mag_j2000[*,1],/NAN,/DOUBLE)
    	bz_mean = MEAN(mag_j2000[*,2],/NAN,/DOUBLE)
    	;btot = SQRT(bx_mean^2 + by_mean^2 + bz_mean^2)
    	;mag_unit_v_j2000 = [bx_mean/btot, by_mean/btot, bz_mean/btot]     ;in radians in J2000 frame
    	CSPICE_UNORM, [bx_mean, by_mean, bz_mean], mag_unit_v_j2000, btot
		
    	;compute angles from average mag vector in J2000 frame, 
    	;VSEP doesn't care if one is unit and the rest are not
    	angles = DBLARR(n_mag)
    	FOR n=0,n_mag-1 DO BEGIN
        	angles[n] = CSPICE_VSEP(mag_unit_v_j2000,  REFORM(mag_j2000[n, *]))
    	ENDFOR

    	;Find if average angle is greater than PITCH.magtol
    	mean_angle = MEAN(angles,/DOUBLE,/NAN)
    	max_angle = MAX(angles,/NAN)

    	IF mean_angle*!RADEG GT  magtol THEN BEGIN
        	IF NOT KEYWORD_SET(SILENT) THEN BEGIN
            	PRINT,'<<WARNING>> The mag data variance over the image accumulation time has exceeded '+$
            	STRCOMPRESS(STRING(magtol),/REMOVE_ALL)
            	PRINT,'The mag mean angle is '+$
            	STRCOMPRESS(STRING(mean_angle*!RADEG),/REMOVE_ALL)+' and maximum angle is '+$
                	STRCOMPRESS(STRING(max_angle*!RADEG),/REMOVE_ALL)
            	PRINT,'So the pitch angle flux will not be plotted for '+utc[0]+'!'
        	ENDIF

    	   RETURN,-1
    	ENDIF
    	IF NOT KEYWORD_SET(SILENT) THEN BEGIN
        	PRINT,'The mag mean angle for is '+$
        	STRCOMPRESS(STRING(mean_angle*!RADEG),/REMOVE_ALL)+' and maximum angle is '+$
        	STRCOMPRESS(STRING(max_angle*!RADEG),/REMOVE_ALL)+' for '+utc[0]
    	ENDIF 


    	;find mag vector in inca instrument coordinate frame (not spice)
    	CSPICE_PXFORM, 'J2000', 'CASSINI_MIMI_INCA_LL', image_et, rotate_J20002incall
    	CSPICE_MXV, rotate_J20002incall, mag_unit_v_J2000, mag_unit_v2

    	;find particle direction
    	CSPICE_VMINUS, mag_unit_v2, mag_unit_v2
		
    	;in inca ll frame
    	azdiv = 90./float(imagesize)
    	eldiv = 120./float(imagesize)
    	azhalf = azdiv/2.
    	elhalf = eldiv/2.
    	radius = 1.
    	FOR a=0,imagesize-1 DO BEGIN

        	FOR e=0,imagesize-1 DO BEGIN

            	;Convert the coordinates from latitudinal to rectangular
            	; remember that phi value assume west positive longitude, that is why lon is negated
            	lon_az = -45. + (float(a)*azdiv) + azhalf + az_shift
            	lat_el = -60. + (float(e)*eldiv) + elhalf + el_shift

            	CSPICE_LATREC, 1.0, (-lon_az[0])/!RADEG, lat_el[0]/!RADEG, inca_unit

        	   ;find angle between of vectors, load angle array from bottom to top to match iamge array
            	angle[a,e] = CSPICE_VSEP(inca_unit, mag_unit_v2)*!RADEG
				
        	ENDFOR  ;elevation

    	ENDFOR  ;azimuth  
		
		mangle[k,*,*] = angle[*,*] 
	ENDFOR

RETURN,0
END
