;+
; NAME:
;	MAKE_VM
;
; PURPOSE:
;	Create Virtual Machine application for linux and MAC platforms. Compiles all code, makes a save set,
;	then runs MAKE_VM on it and creates a directory application_<version>_<platform> in dirstub. Keywords
;	are included to allow additional files to be copied to the directory.
;   
; NOTE: 
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	MAKE_VM, idlprogram, application, dirstub
;
; EXAMPLES:
;   For kd_cross_sections_tool, use the following. The directory will be 
;       cross_sections_0.0_linux/cross_sections
;
;   MAKE_VM,'kd_cross_sections_tool','cross_sections','~',$
;               version='1.2',requiredfiles=['*.txt','*.text','README']
;
; NOTES:
;	For Unix/linux, just run make_vm, tar and zip the resultant directory and people can just unzip and untar
;		and run it.
;
;	For MAC, run make_vm, which makes the directory and copies IDL_DIR/bin/make_rt/start_app_mac.app, renames it 
;		to dirstub.app and copies over IDL_DIR/bin/make_rt/Utils_applescripts.scpt as well. Then the user
;		should just be able to click on this.
;
; INPUTS:
;   application Name of application, will create a directory in dirstub, _platform name will be
;               added to the end of the directory
;   dirstub      Directory where application will be placed
;
; KEYWORD PARAMETERS:
;   VERSION     Set this to the string software version. It will be included in the path name
;   REQUIREDFILES  Set this to an array of string files in idlprogram's directory
;               that are needed in addition to the IDL save set. Example: ['*.txt','*.text','README']
;
;                       Copyright (C) 2009
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       22 Sep 2009, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;       10 Dec 2009, M.B. Kusterer 	Took out MAKE_RT since it no longer works with IDL7.1
;       22 2010, M.B. Kusterer 	Made use version environmental variable to name save set so can
;		have multiple versions. Also corrected darwin name, is all low case in IDL.
;       May 12 2011, M.B. Kusterer Added FORPDS keyword.
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO MAKE_VM, applicationname, dirstub, VERSION=VERSION, $
    REQUIREDFILES=REQUIREDFILES, NOCHECK=NOCHECK, FORPDS=FORPDS, $
    _REF_EXTRA=keyword_ex

IF NOT KEYWORD_SET(NOCHECK) THEN BEGIN
    answer = ''
    PRINT,'Did you run a .FULL_RESET_SESSION before starting this routine?'
    PRINT,'Enter a yes, (y) or no (n):'
    READ,answer
    at = STRPOS(answer,'y')
    IF at[0] EQ -1 THEN BEGIN
        PRINT,'Then run .FULL_RESET_SESSION and retry this routine'
        RETURN
    ENDIF
ENDIF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Create and check dirstub
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IF N_ELEMENTS(VERSION) GT 0 THEN BEGIN
    strver = '_'+VERSION
ENDIF ELSE strver = ''

platform = !VERSION.OS

;correct linux text name
IF STRMATCH(platform,'linux') THEN platform = 'Linux'
IF STRMATCH(platform,'darwin') THEN platform = 'Darwin'

apdir = applicationname+strver+'_'+platform
newdir = dirstub+PATH_SEP()+apdir

found = FILE_TEST(newdir,/DIRECTORY)
IF found NE 1 THEN BEGIN
    FILE_MKDIR,newdir
ENDIF

found = FILE_TEST(newdir,/DIRECTORY,/WRITE)
IF found NE 1 THEN BEGIN
    PRINT,'Failure to make '+newdir
    PRINT,'Exiting MAKE_VM'
    RETURN
ENDIF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Compile and make save set for applicationname
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;compile all code in dir
print,'MAKE_VM: Compiling routines ...'
IF KEYWORD_SET(FORPDS) THEN BEGIN
	RESOLVE_ROUTINE,'pds_read_mimi'
	RESOLVE_ROUTINE,'pds_read_mimiimg'
	RESOLVE_ROUTINE,'spice_et2utc'
	RESOLVE_ROUTINE,'spice_utc2et'
ENDIF ELSE BEGIN
	RESOLVE_ROUTINE,'get_data'
ENDELSE
COMPILE_ALL,applicationname, CODEDIR=CODEDIR,_EXTRA=['SKIP_ROUTINES','SKIP_DIR']

;print,'Out of compile_all into resolve_routine'
;RESOLVE_ALL,RESOLVE_FUNCTION ='map_image'
;RESOLVE_ALL,RESOLVE_FUNCTION ='bin_date'
;RESOLVE_ALL,RESOLVE_PROCEDURE ='expand'

print,'MAKE_VM: Making Saveset...'


saveset = newdir+PATH_SEP()+applicationname+'.sav'
SAVE,/ROUTINES,FILENAME=saveset,/VERBOSE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;call make_rt for the saveset
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;print,'MAKE_VM: Making Real Time Application...'
;IF STRMATCH(platform,'linux') EQ 1 THEN BEGIN
;    MAKE_RT, applicationname, newdir, SAVEFILE=saveset,/VM,/lin32,/OVERWRITE
;    
;ENDIF ELSE IF STRMATCH(platform,'darwin') EQ 1 THEN BEGIN
;    MAKE_RT, applicationname, newdir, SAVEFILE=saveset,/VM,/macint32,/macppc32,/OVERWRITE
;    
;ENDIF ELSE BEGIN    ;PC I think
;    MAKE_RT, applicationname, newdir, SAVEFILE=saveset,/VM,/OVERWRITE
;
;ENDELSE


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;copy over required data files
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IF N_ELEMENTS(REQUIREDFILES) GT 0 THEN BEGIN
print,'MAKE_VM: Copying over required files...'
    FOR n=0,N_ELEMENTS(REQUIREDFILES)-1 DO BEGIN
        t = FILE_TEST(CODEDIR+PATH_SEP()+REQUIREDFILES[n],/READ)
        IF t EQ 1 THEN BEGIN
		print,'1st ',CODEDIR+PATH_SEP()+REQUIREDFILES[n]
		print,'2nd ',newdir+PATH_SEP()
            FILE_COPY,CODEDIR+PATH_SEP()+REQUIREDFILES[n],newdir+PATH_SEP(),/OVERWRITE
        ENDIF
    ENDFOR
ENDIF


print,'Done, created '+applicationname+' in '+newdir+' directory'

RETURN
END
