;+
; NAME:
;	PDS_CALIBRATE_IMAGES
;
; PURPOSE:
;	To  calibrate PDS mimi inca image data. 
;   
; NOTE: 
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	struct = PDS_CALIBRATE_IMAGES(starttime, stoptime, imagetype, arfiles, $
;			units,  data, images)
;
; INPUTS:
;	starttime, stoptime, yyyy-doyThh:mm:ss.msc start and stop UTC strings
;	imagetype				String image resolution type flag, 
;								'Spatial': High Spatial
;								'Time':    High Time
;								'm-TOF':   High m-TOF
;							High Spatial resolution images with tof=0 have 64x64 pixels, tof=7 have 32,32 pixels
;							They have species H=hydrogen and CNO=Oxygen
;							High Time resoluton images have 32x32 pixels and have species All
;							High m-TOF resoluton images have 16x16 pixels.
;							They have species H=hydrogen and CNO=Oxygen and have TOF 0-7
;   arfiles                 Input path and PDS MIMI INCA Accumulator Rates filenames 
;							with extention. Needed for inca calibration.
;	units					Default units is counts
;							'Counts'	Counts
;							'Cnts/sec'  Counts per second
;							'Int_Flux'	Integral Intensity
;							'Dif_Flux'  Differential Intensity
;   data	        array size [numrows,numcols]
;	images			array size [numrows, maximagesize, maximagesize]

;
; OUTPUTS:
;	images			Will contain calibrated images
;   bandwidth       Double array containing image energy bandwidth [numrows]
;   energyrange     Double array containing image energy range [2, numrows]
;
; KEYWORD PARAMETERS:
;
;                       Copyright (C) 2010
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       01 Oct 2010, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO PDS_CALIBRATE_IMAGES, starttime, stoptime, imagetype, arfiles, $
			units, data, images, bandwidth, energyrange
		
@pds_data_hdr_h     ;include data header column indexes
@pds_image_hdr_h     ;include inca image header column indexes	
			
	;This parameter is a remnant of an old flight software mode that we rarely used
	;It needs to stay equal to 0 since I do not think we are putting any of the 
	;images in the pds from the other mode. This is NOT the same as the charge_c parameter
	; which distinguishes between the ion and neutral particle images.
	commandedcharge = 0
			
	usecal = 0
	IF STRLOWCASE(units) EQ 'counts' THEN BEGIN
		usecal = 0
		RETURN
    ENDIF ELSE IF STRLOWCASE(UNITS) EQ 'counts/s' THEN BEGIN 
		usecal = 1
    ENDIF  ELSE IF STRLOWCASE(UNITS) EQ 'counts/(cm*s*sr)' THEN BEGIN 
		usecal = 2
    ENDIF  ELSE IF STRLOWCASE(UNITS) EQ 'counts/(cm*s*sr*kev)' THEN BEGIN
		usecal = 3
    ENDIF  ELSE BEGIN
		PRINT,'PDS_CALIBRATE_IMAGES: Units value was illegal, returning counts'
		RETURN
    ENDELSE 
	
	print,'Usecal ',usecal
	
	numcols = N_ELEMENTS(data[0,*])
	numrows = N_ELEMENTS(data[*,0])
	maximagesize = N_ELEMENTS(images[0,*,0])

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Read in INCA Accumulator Rates Files if flux is specified
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	have_ar_ratio=0
	IF usecal GT 1 AND N_ELEMENTS(arfiles) GT 0 THEN BEGIN
		IF arfiles[0] NE '' THEN BEGIN
			s=UTC2ET([starttime, stoptime],et)
			;widen time range so get all of science record
			et[0] = et[0]-20.
			et[1] = et[1]+18*60.
			s=ET2UTC(et,utc)

			TIMESTR2NUM, utc[0], startyear, startdoy, starthours, startmins, startsecs
			TIMESTR2NUM, utc[1], stopyear, stopdoy, stophours, stopmins, stopsecs

			s=PDS_READ_MIMI(startyear, startdoy, starthours, startmins, startsecs, $
                        	   stopyear, stopdoy, stophours, stopmins, stopsecs, arfiles, $
                        	   ar_labels, ar_data, ar_datautc)
			have_ar_ratio = 1
			IF s EQ -1 THEN BEGIN
				have_ar_ratio = 0
			ENDIF
			;locations of acc rates Time and Events_recieved parameters
			timeat = WHERE(STRMATCH(ar_labels,'Time') EQ 1)
			eventrec = WHERE(STRMATCH(ar_labels,'Events_Received') EQ 1)
			timeat = timeat[0]
			eventrec = eventrec[0]
			IF timeat[0] EQ -1 OR eventrec[0] EQ -1 THEN BEGIN
				have_ar_ratio = 0
			ENDIF
		ENDIF ELSE BEGIN
			have_ar_ratio = 0
		ENDELSE
	ENDIF 
	IF usecal GE 2 AND (N_ELEMENTS(arfiles) EQ 0 OR have_ar_ratio EQ 0) THEN BEGIN
		PRINT,'PDS_CALIBRATE_IMAGES: Failure to find matching Inca Accumulator Rates data '
		PRINT,'So the accumulator rates ratio will not be applied to the image intensity calculations'
		have_ar_ratio = 0
	ENDIF

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; Read in Calibration Files if needed
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	calstatus = PDS_FIND_INCA_OUTOFCAL(starttime, stoptime)
	

	s=PDS_READ_IMAGECAL(starttime, fill_data, imagetype, usecal,  $
		energy_widths, backgrounds, calib, energy_ranges)

	factor = 1./16.

	FOR n=0l,numrows-1 DO BEGIN

		;figure out accumulation time in seconds, remember to *4 for quad images
		timeofone = DOUBLE(data[n,SPIN_PERIOD_C_C]) * factor
		quad = 1d
		IF data[n,TYPE_C] EQ IMAGETYPE_HIGHSPATIAL OR data[n,TYPE_C] EQ IMAGETYPE_HIGHMTOF THEN BEGIN
			quad = 4d
		ENDIF
		accum_secs = quad*timeofone

		imagesize = data[n,NUMCOLS_C]
		spinmode = data[n,SPINSTAR_MODE_C]
		type = data[n,TYPE_C]
		specie = data[n,SPECIE_C]
		tof = data[n,TOF_C]

		IF usecal EQ -1 OR usecal GE 1 THEN BEGIN

			IF (type EQ IMAGETYPE_HIGHSPATIAL) THEN BEGIN
            	IF (specie EQ IMAGESPECIE_H) THEN BEGIN
                	IF (TOF EQ IMAGETOF_LOW) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hispat_loTOF_neu_H_caldata : $
													calib.hispat_loTOF_neu_H_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_spat_LowTOF_H
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_spat_LowTOF_H
						bw = energy_widths.high_spat_LowTOF_H
						er = energy_ranges.high_spat_LowTOF_H
                	ENDIF ELSE IF (TOF EQ IMAGETOF_HIGH) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hispat_hiTOF_neu_H_caldata : $
													calib.hispat_hiTOF_neu_H_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_spat_HighTOF_H
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_spat_HighTOF_H
						bw = energy_widths.high_spat_HighTOF_H
						er = energy_ranges.high_spat_HighTOF_H
                	ENDIF
            	ENDIF ELSE IF (specie EQ IMAGESPECIE_CNO) THEN BEGIN
                	IF (TOF EQ IMAGETOF_LOW) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hispat_loTOF_neu_O_caldata : $
													calib.hispat_loTOF_neu_O_caldata_spin 

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_spat_LowTOF_O
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_spat_LowTOF_O
						bw = energy_widths.high_spat_LowTOF_O
						er = energy_ranges.high_spat_LowTOF_O
                	ENDIF ELSE IF (TOF EQ IMAGETOF_HIGH) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hispat_hiTOF_neu_O_caldata : $
													calib.hispat_hiTOF_neu_O_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_spat_HighTOF_O
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_spat_HighTOF_O
						bw = energy_widths.high_spat_HighTOF_O
						er = energy_ranges.high_spat_HighTOF_O
                	ENDIF
            	ENDIF

        	ENDIF ELSE IF (type EQ IMAGETYPE_HIGHTIME) THEN BEGIN
				IF commandedcharge EQ 0 THEN BEGIN
                	IF (TOF EQ IMAGETOF_LOW) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hitime_loTOF_neu_caldata : $
													calib.hitime_loTOF_neu_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_time_LowTOF_neu
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_time_LowTOF[commandedcharge]
						bw = energy_widths.high_time_LowTOF_neu
						er = energy_ranges.high_time_LowTOF_neu
                	ENDIF ELSE IF (TOF EQ IMAGETOF_HIGH) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hitime_hiTOF_neu_caldata : $
													calib.hitime_hiTOF_neu_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_time_HighTOF_neu
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_time_HighTOF[commandedcharge]
						bw = energy_widths.high_time_HighTOF_neu
						er = energy_ranges.high_time_HighTOF_neu
                	ENDIF
				ENDIF ELSE BEGIN
                	IF (TOF EQ IMAGETOF_LOW) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hitime_loTOF_ion_caldata : $
													calib.hitime_loTOF_ion_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_time_LowTOF_ion
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_time_LowTOF[commandedcharge]
						bw = energy_widths.high_time_LowTOF_ion
						er = energy_ranges.high_time_LowTOF_ion
                	ENDIF ELSE IF (TOF EQ IMAGETOF_HIGH) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
                    		geoeff = (spinmode EQ 0) ? calib.hitime_hiTOF_ion_caldata : $
													calib.hitime_hiTOF_ion_caldata_spin

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_time_HighTOF_ion
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_time_HighTOF[commandedcharge]
						bw = energy_widths.high_time_HighTOF_ion
						er = energy_ranges.high_time_HighTOF_ion
                	ENDIF
				ENDELSE

        	ENDIF ELSE IF (type EQ IMAGETYPE_HIGHMTOF) THEN BEGIN
				IF commandedcharge EQ 0 THEN BEGIN
            		IF (specie EQ IMAGESPECIE_H) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
            		   		geoeff = (spinmode EQ 0) ? REFORM(calib.himTOF_neu_H_caldata[tof,*,*]) : $
					   							REFORM(calib.himTOF_neu_H_caldata_spin[tof,*,*])

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_mTOF_H_neu[tof]
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_mTOF_H[commandedcharge, tof]
						bw = energy_widths.high_mTOF_H_neu[tof]
						er = energy_ranges.high_mTOF_H_neu[*,tof]
            		ENDIF ELSE IF (specie EQ IMAGESPECIE_CNO) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
            		   		geoeff = (spinmode EQ 0) ? REFORM(calib.himTOF_neu_O_caldata[tof,*,*]) : $
					   							REFORM(calib.himTOF_neu_O_caldata_spin[tof,*,*])

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_mTOF_O_neu[tof]
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_mTOF_O[commandedcharge, tof]
						bw = energy_widths.high_mTOF_O_neu[tof]
						er = energy_ranges.high_mTOF_O_neu[*,tof]
            		ENDIF
				ENDIF ELSE BEGIN
            		IF (specie EQ IMAGESPECIE_H) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
            		   		geoeff = (spinmode EQ 0) ? REFORM(calib.himTOF_ion_H_caldata[tof,*,*]) : $
					   							REFORM(calib.himTOF_ion_H_caldata_spin[tof,*,*])

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_mTOF_H_ion[tof]
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_mTOF_H[commandedcharge, tof]
						bw = energy_widths.high_mTOF_H_ion[tof]
						er = energy_ranges.high_mTOF_H_ion[*,tof]
            		ENDIF ELSE IF (specie EQ IMAGESPECIE_CNO) THEN BEGIN
						IF usecal EQ -1 OR usecal GE 2 THEN BEGIN
            		   		geoeff = (spinmode EQ 0) ? REFORM(calib.himTOF_ion_O_caldata[tof,*,*]) : $
					   							REFORM(calib.himTOF_ion_O_caldata_spin[tof,*,*])

							ew = (usecal EQ -1) ? 0.0 : energy_widths.high_mTOF_O_ion[tof]
						ENDIF
						bkgrnd = (usecal EQ -1) ? 0.0 : backgrounds.high_mTOF_O[commandedcharge, tof]
						bw = energy_widths.high_mTOF_O_ion[tof]
						er = energy_ranges.high_mTOF_O_ion[*,tof]
            		ENDIF
				ENDELSE
				
        	ENDIF
			
			;load bandwidths and energy_ranges
			bandwidth[n] = bw
			energyrange[*,n] = er

			;efficiency and geometric factor matrix,
			IF usecal EQ -1 THEN BEGIN
				images[n, 0:imagesize-1, 0:imagesize-1] = geoeff[0:imagesize-1, 0:imagesize-1]

			;counts per second
			ENDIF ELSE IF usecal EQ 1 THEN BEGIN
				images[n, 0:imagesize-1, 0:imagesize-1] = $
					(images[n, 0:imagesize-1, 0:imagesize-1]/FLOAT(accum_secs)) - bkgrnd

			;integral flux = image*(geometricfactor and efficiency)/accumulation time
			ENDIF ELSE IF usecal EQ 2 THEN BEGIN
				;find matching accumulator rates data and calculate ratio
				IF have_ar_ratio EQ 1 THEN BEGIN
					at = (quad EQ 4d) ? WHERE(ar_data[*,SPIN_COUNTER_C] EQ data[n,SPIN_COUNTER_C] AND $
										(ar_data[*,SECTOR_NUMBER_C] GE data[n,SECTOR_NUMBER_C] AND $
										ar_data[*,SECTOR_NUMBER_C] LT data[n,SECTOR_NUMBER_C]+4),count): $
										WHERE(ar_data[*,SPIN_COUNTER_C] EQ data[n,SPIN_COUNTER_C] AND $
										ar_data[*,SECTOR_NUMBER_C] EQ data[n,SECTOR_NUMBER_C],count)

					ratio = (at[0] NE -1) ? (TOTAL(ar_data[at,timeat])/count)/(TOTAL(ar_data[at,eventrec])/count) : 1.0
				ENDIF ELSE ratio = 1.0

				images[n, 0:imagesize-1, 0:imagesize-1] = $
					images[n, 0:imagesize-1, 0:imagesize-1]*geoeff*ratio/DOUBLE(accum_secs) - bkgrnd
					



			;differential flux = image*(geometricfactor and efficiency)/accumulation time/energy with
			ENDIF ELSE IF usecal EQ 3 THEN BEGIN
				IF have_ar_ratio EQ 1 THEN BEGIN
					at = (quad EQ 4d) ? WHERE(ar_data[*,SPIN_COUNTER_C] EQ data[n,SPIN_COUNTER_C] AND $
										(ar_data[*,SECTOR_NUMBER_C] GE data[n,SECTOR_NUMBER_C] AND $
										ar_data[*,SECTOR_NUMBER_C] LT data[n,SECTOR_NUMBER_C]+4),count): $
										WHERE(ar_data[*,SPIN_COUNTER_C] EQ data[n,SPIN_COUNTER_C] AND $
										ar_data[*,SECTOR_NUMBER_C] EQ data[n,SECTOR_NUMBER_C],count)

					ratio = (at[0] NE -1) ? (TOTAL(ar_data[at,timeat])/count)/(TOTAL(ar_data[at,eventrec])/count) : 1.0
				ENDIF ELSE ratio = 1.0

				images[n, 0:imagesize-1, 0:imagesize-1] = $
					images[n, 0:imagesize-1, 0:imagesize-1]*geoeff*ratio/accum_secs/ew - bkgrnd

			ENDIF
		ENDIF


	ENDFOR	
RETURN
END
