;+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Program Name:	PDS_FIND_INCA_OUTOFCAL
;
; Description:	Reads the Cassini/MIMI INCA hskp high voltage values to 
;				indicate when the instrument is out of calibration.
;
; CALLING SEQUENCE:
;	calstatus = PDS_FIND_INCA_OUTOFCAL(starttime, stoptime)
;
; NOTE:
;	This routine depends on the environmental variable OUTOFCAL_DIR being set to the 
;	directory of the  MIMI INCA High Voltage Out of Calibration files. This is done in
;	the setup.pro routine. The user should edit the setup file to point to their paths.
;	See MIMI website for files
;					http://cassini-mimi.jhuapl.edu/data_auxiliary.html
;
; Input:   
;   starttime	    text UTC starttime
;   stoptime		text UTC stoptime
;   dir				directory to inca outof cal files, needs path separator at end, 
;					
;
; Output:
;	CALSTATUS	Returns a structure {outofcal:0-not, 1-yes, 
;				etranges:STRARR(2,*),utcranges:STRARR(2,*)} list of utc
;
; Copyright (C) 2010, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;       	
; Change History:
;
; Date		Rev	Programmer	Comments
; ----		---	----------	--------
;       09 May 2010, M.B. Kusterer  Original
;       09 May 2010, M.B. Kusterer  Added PDS_ to name, and replaced dir with OUTOFCAL_DIR env var
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_FINDOUTCAL

starttime = '2009-001T00:00'
stoptime = '2009-020T23:59:59.999'
;starttime = '2004-057T15:40:00.000'
;stoptime = '2004-059T21:00:00.000'

calstatus = PDS_FIND_INCA_OUTOFCAL(starttime, stoptime)

help,calstatus,/struct

RETURN
END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION PDS_FIND_INCA_OUTOFCAL, starttime, stoptime


    status = 0
	outofcal = 0
	utcranges = 0
	etranges = 0
	ntags = 0
	
    dir = GETENV('OUTOFCAL_DIR')
	
    status = PDS_READ_OUTOFCAL(starttime, stoptime, dir, caldata, cstatus)
    IF status EQ -1 THEN RETURN,status
    
    ;set outofcal if have any ranges out of calibration
    outofcal = (cstatus NE -1) ? 1 : 0
    
    ;set utcranges and etranges 
    utcranges = ''
    etranges = 0d
    calflag = 0
    nelem = N_ELEMENTS(caldata)
    IF cstatus NE -1 THEN BEGIN
        utcranges = STRARR(2,nelem)
        etranges = DBLARR(2,nelem)
        FOR n=0,nelem-1 DO BEGIN
            utcranges[0,n] = caldata[n].startutc
            utcranges[1,n] = caldata[n].stoputc
            etranges[0,n] = caldata[n].startet
            etranges[1,n] = caldata[n].stopet
        ENDFOR
    ;set calflag to allert which type of out of calibration state
        calflag = [caldata[*].flag]
        
		mess = '<<WARNING>> Inca high voltages are out of Calibration for the following UTC ranges'
        flag = ['','Planned','Unplanned']
		FOR n=0L,N_ELEMENTS(utcranges[0,*])-1 DO mess =  [mess,utcranges[0,n]+' - '+utcranges[1,n]+$
                                                            ' '+flag[calflag[n]]]
        PRINT,mess
			
    ENDIF
    
	CALSTATUS = {outofcal:outofcal, utcranges:utcranges, etranges:etranges,$
                calflag:calflag,cstatus:cstatus}
	
RETURN, calstatus
END
