;+
; NAME:	
;	PDS_LOAD_COLORMAP
;
; PURPOSE:
;	This procedure loads a colormap using the colormapsize minus 12 places for text colors.
;   It assumes that either a window or a postscript device is open
;
; CATEGORY:
;	Plotting setup routine
;
; CALLING SEQUENCE:
;	LOAD_COLORMAP,colormap,colormapsize
;
; PARAMETERS:
;   colormap        
;   colormapsize    size of color map indexes
;
; KEYWORD PARAMETERS:
;   WHITEBACK   For GIF,PNG and JPEG background color will be white when this is set
;   only if PS is not the device!
;
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
;
; CREATION:
;       21 August 1997, M.B. Kusterer      
;
; MODIFICATION HISTORY:
;       12 Aug 2002, M.B. Kusterer Added WHITEBACK keyword
;       5 Feb 2003, M.B. Kusterer   Fixed problem with a super huge device colormap. I limit
;           it to 256.
;       1 April 2003, M.B. Kusterer  Made 0 index same color as background, white or black. Changed PS to work the
;                       same as png, and jpeg
;       17 May 2004, M.B. Kusterer  Corrected change to colormapsize so the top color on major map is correct.
;       8 Jun 2005, M.B. Kusterer  Added noerase keyword
;       23 Sep 2005, M.B. Kusterer Found that colormaps 39, 40 have white and black at the top respectively.
;       Will change top color of colormaps 39,40.
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO PDS_LOAD_COLORMAP, colormap, colormapsize, WHITEBACK=WHITEBACK, NOERASE=NOERASE

     
     
    ;Limit device colormap to 256
    IF !D.N_COLORS GT 256 THEN device_colormapsize = 256 $
    ELSE device_colormapsize = !D.N_COLORS

    numcolors = 12
   
	LOADCT,colormap	
    
	;correct colormap size to that allowed on window or PS device
  	colormapsize = device_colormapsize-numcolors-2
    
	smallnum = device_colormapsize-numcolors
    
    ;get the red green and blue color values from current color map
   	TVLCT, red, green, blue, /GET
    n = N_ELEMENTS(red)
    IF colormap EQ 39 OR colormap Eq 40 THEN BEGIN
        red[n-1] = red[n-2]
        green[n-1] = green[n-2]
        blue[n-1] = blue[n-2]
    ENDIF
   
    smred = CONGRID(red,smallnum)
    smgreen = CONGRID(green,smallnum)
    smblue = CONGRID(blue,smallnum)
    n=n_elements(smblue)
       
    IF KEYWORD_SET(WHITEBACK) THEN BEGIN           
   	    smred =   [smred,  30,  78,  0,  150,  112,  255,   0, 255, 254,  254, 0, 255]
   	    smgreen = [smgreen,255, 180, 173, 150, 124,  0,   255,  0,   140, 255, 0, 255]
   	    smblue =  [smblue, 255,  0,   255, 150, 254,  255,  0,   0,   0,   0,   0, 255]
        smred[0] = 255    ;load white for zero index
        smgreen[0] = 255
        smblue[0] = 255 
    ENDIF ELSE BEGIN                                         
   	    smred =   [smred,  30,  78,  0,  150,  112,  255,   0, 255, 254,  254, 255, 0]
   	    smgreen = [smgreen,255, 180, 173, 150, 124,  0,   255,  0,   140, 255, 255, 0]
   	    smblue =  [smblue, 255,  0,   255, 150, 254,  255,  0,   0,   0,   0,   255, 0]
        smred[0] = 0    ;load black for zero index
        smgreen[0] = 0
        smblue[0] = 0
    ENDELSE                                 ;white back  black back
	!P.background = N_ELEMENTS(smblue)-1    ;white        black
	!P.color = N_ELEMENTS(smblue)-2         ;black         white
    TVLCT, smred, smgreen, smblue
    
    IF NOT KEYWORD_SET(NOERASE) THEN BEGIN
    ;print,'erasing from load_colormap'
        ERASE
    
        IF NOT KEYWORD_SET(WHITEBACK) THEN BEGIN  
            ;print,'polyfilling background from load_colormap '
            
            POLYFILL,[1,1,0,0,1],[1,0,0,1,1],/NORMAL,COLOR=N_ELEMENTS(smblue)-1
        ENDIF
    ENDIF
   	
RETURN
END
