;+
; NAME:
;	PDS_READ_IMAGECAL
;
; PURPOSE:
;	Reads in all inca image calibration information for a utc time

; CATEGORY:
;	Support IO Routine. 
;
; NOTE:
;	This routine depends on the environmental variable INCAFLUX_DIR being set to the 
;	directory of the  MIMI INCA Flux directory set geometricfactorXefficiency_matrix. This is done in
;	the setup.pro routine. The user should edit the setup file to point to their paths.
;	See MIMI website for files
;					http://cassini-mimi.jhuapl.edu/data_auxiliary.html
;
; CALLING SEQUENCE:
;	s=PDS_READ_IMAGECAL, strtime, fill_data, imagetype, usecal, calibdir, $
;		energy_widths, backgrounds, calib, energy_ranges
;
;   Uses env var INCAFLUX_DIR to get calibration filename, this is set in the setup.pro file.
;   Obtain files from "http://cassini-mimi.jhuapl.edu/data_auxiliary.html"
;
; INPUTS:
;	strtime	UTC time, yyyy-doyThh:mm:ss.msc
;	fill_data	The fill data is set depending on the level of calibration
;
;	imagetype				String image resolution type flag, 
;								'Spatial':High Spatial, 'Time':High Time, 'm-TOF': High m-TOF
;							High Spatial resolution images with tof=0 have 64x64 pixels, tof=7 have 32,32 pixels
;							They have species H=hydrogen and CNO=Oxygen
;							High Time resoluton images have 32x32 pixels and have species All
;							High m-TOF resoluton images have 16x16 pixels.
;							They have species H=hydrogen and CNO=Oxygen and have TOF 0-7
;	usecal			Calibration flag, 0-counts, 1-cnts/sec, 2=integral intensity ,3=differential intensity
;	fluxcalibdir			String path to where the MIMI INCA Flat Fielding file directory set was unzipped. 
;							Obtain gzipped tar set from 
;							"http://cassini-mimi.jhuapl.edu/data_auxiliary.html". 
;	mimicalibdir			String path to where the MIMI calibration file is located. Obtain file from 
;							"http://cassini-mimi.jhuapl.edu/data_auxiliary.html". 
;
; OUTPUTS:
;	energy_widths	Structure from READ_INCA_BW_WIDTHS
;						energy_widths = {$
;						high_time_LowTOF_neu:0.,$
;						high_time_HighTOF_neu:0.,$
;						high_time_LowTOF_ion:0.,$
;                        high_time_HighTOF_ion:0.,$
;                        high_spat_LowTOF_H:0.,$
;                        high_spat_HighTOF_H:0.,$
;                        high_spat_LowTOF_O:0.,$
;                        high_spat_HighTOF_O:0.,$
;                        high_mTOF_H_neu:FLTARR(8),$
;                        high_mTOF_O_neu:FLTARR(8),$
;                        high_mTOF_H_ion:FLTARR(8),$
;                        high_mTOF_O_ion:FLTARR(8)$
;                     }
;	backgrounds    structure from READ_INCA_BKGROUND
;						backgrounds = {$
;                        high_time_LowTOF:FLTARR(2),$    ;[0] neutral, [1] ion
;                        high_time_HighTOF:FLTARR(2),$;[0] neutral, [1] ion
;                        high_spat_LowTOF_H:0.0,$
;                        high_spat_HighTOF_H:0.0,$
;                        high_spat_LowTOF_O:0.0,$
;                        high_spat_HighTOF_O:0.0,$
;                        high_mTOF_H:FLTARR(2,8),$;[0,*] neutral, [1] ion
;                        high_mTOF_O:FLTARR(2,8)$;[0,*] neutral, [1] ion }
;   calib = structure from CALIBRATE_PDS_IMAGEs
;			{hispat_loTOF_neu_H_caldata:DBLARR[64,64],$
;			hispat_loTOF_neu_O_caldata:DBLARR[64,64],$
;			hispat_hiTOF_neu_H_caldata:DBLARR[32,32],$
;			hispat_hiTOF_neu_O_caldata:DBLARR[32,32],$
;			hispat_loTOF_neu_H_caldata_spin:DBLARR[64,64],$
;			hispat_loTOF_neu_O_caldata_spin:DBLARR[64,64],$
;			hispat_hiTOF_neu_H_caldata_spin:DBLARR[32,32],$
;			hispat_hiTOF_neu_O_caldata_spin:DBLARR[32,32],$
;			hitime_loTOF_neu_caldata:DBLARR[32,32],$  8
;			hitime_hiTOF_neu_caldata:DBLARR[32,32],$
;			hitime_loTOF_ion_caldata:DBLARR[32,32],$
;			hitime_hiTOF_ion_caldata:DBLARR[32,32],$
;			hitime_loTOF_neu_caldata_spin:DBLARR[32,32],$
;			hitime_hiTOF_neu_caldata_spin:DBLARR[32,32],$
;			hitime_loTOF_ion_caldata_spin:DBLARR[32,32],$
;			hitime_hiTOF_ion_caldata_spin:DBLARR[32,32],$	15
;			himTOF_neu_H_caldata:DBLARR[8,16,16],$
;			himTOF_neu_O_caldata:DBLARR[8,16,16],$
;			himTOF_ion_H_caldata:DBLARR[8,8,8],$
;			himTOF_ion_O_caldata:DBLARR[8,8,8],$
;			himTOF_neu_H_caldata_spin:DBLARR[8,16,16],$
;			himTOF_neu_O_caldata_spin:DBLARR[8,16,16],$
;			himTOF_ion_H_caldata_spin:DBLARR[8,8,8],$
;			himTOF_ion_O_caldata_spin:DBLARR[8,8,8]}
;    energy_ranges = {$
;                        high_time_LowTOF_neu:FLTARR(2),$
;                        high_time_HighTOF_neu:FLTARR(2),$
;                        high_time_LowTOF_ion:FLTARR(2),$
;                        high_time_HighTOF_ion:FLTARR(2),$
;                        high_spat_LowTOF_H:FLTARR(2),$
;                        high_spat_HighTOF_H:FLTARR(2),$
;                        high_spat_LowTOF_O:FLTARR(2),$
;                        high_spat_HighTOF_O:FLTARR(2),$
;                        high_mTOF_H_neu:FLTARR(2,8),$
;                        high_mTOF_O_neu:FLTARR(2,8),$
;                        high_mTOF_H_ion:FLTARR(2,8),$
;                        high_mTOF_O_ion:FLTARR(2,8)$
;                     }
; KEYWORD PARAMETERS:
;   
;
;                       Copyright (C) 2010
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       14 Feb 2011, M.B. Kusterer 
;
; MODIFICATION HISTORY:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_RIC,energy_widths,backgrounds,calib

;this test routine only works inside MIMI team machines, it is for comparing
;between mimi software and mimi pds software
	utc = '2009-365T00:00:00'
	utc = '2003-263T00:00:00'
	utc = '2010-200T10:30:00'
	p=PATH_SEP()
	;imagetype = 'Spatial'
	imagetype = 'm-TOF'
	;imagetype = 'Time'
	usecal = 1
	usecal = 2
	usecal = 3
	
	s=PDS_READ_IMAGECAL(utc, fill, imagetype, usecal,  $
		energy_widths, backgrounds, calib, energy_ranges)
		
	print,energy_widths
	
RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION READMATRIX, file, msize, matrix

	matrix = DBLARR(msize,msize)
	OPENR,lun,file,/GET_LUN,ERROR=err
	IF err LT 0 THEN BEGIN
		PRINT,'READMATRIX: Failure to open file '+file
		RETURN,-1
	ENDIF
	READF,lun,matrix
	
	CLOSE,lun
	FREE_LUN,lun
	
RETURN,0
END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FUNCTION PDS_READ_IMAGECAL, strtime, fill_data, imagetype, usecal, $
                        energy_widths, backgrounds, calib , energy_ranges
		
	
	energy_widths = 0
	backgrounds = 0
	calib = 0
	hispat_loTOF_neu_H_caldata = 0
	hispat_loTOF_neu_O_caldata = 0
	hispat_hiTOF_neu_H_caldata = 0
	hispat_hiTOF_neu_O_caldata = 0
	hispat_loTOF_neu_H_caldata_spin = 0
	hispat_loTOF_neu_O_caldata_spin = 0
	hispat_hiTOF_neu_H_caldata_spin = 0
	hispat_hiTOF_neu_O_caldata_spin = 0
	hitime_loTOF_neu_caldata = 0
	hitime_hiTOF_neu_caldata = 0
	hitime_loTOF_ion_caldata = 0
	hitime_hiTOF_ion_caldata = 0
	hitime_loTOF_neu_caldata_spin = 0
	hitime_hiTOF_neu_caldata_spin = 0
	hitime_loTOF_ion_caldata_spin = 0
	hitime_hiTOF_ion_caldata_spin = 0
	himTOF_neu_H_caldata = 0
	himTOF_neu_O_caldata = 0
	himTOF_ion_H_caldata = 0
	himTOF_ion_O_caldata = 0
	himTOF_neu_H_caldata_spin = 0
	himTOF_neu_O_caldata_spin = 0
	himTOF_ion_H_caldata_spin = 0
	himTOF_ion_O_caldata_spin = 0
	
	
	p=PATH_SEP()
	s=UTC2ET(strtime,et2use)
	
	;set fill value
	fill_data = 65535d
	IF usecal EQ -1 OR usecal GT 0 THEN fill_data = 1.7976900D+308
	
		
	IF usecal GE 1 THEN BEGIN
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;get image bandwidths
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		backgrounds=0
    	IF usecal NE -1 THEN BEGIN
    		s = PDS_READ_INCA_BKGROUND(et2use, backgrounds, usecal)
			IF s LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to get image backgrounds '
				RETURN,-1
			ENDIF
		ENDIF
	ENDIF
	
	;read in calibration matrix
	IF usecal EQ -1 OR usecal GT 1 THEN BEGIN
	print,'In calibration'
							
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;get image bandwidths
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		energy_widths=0
		IF usecal NE -1 THEN BEGIN
    		s = PDS_READ_INCA_BW(et2use, energy_widths, energy_ranges)
			IF s LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to get image bandwidths '
				RETURN,-1
			ENDIF
		ENDIF
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;find what calibration set to use, read decom_incaflux_dates.txt
        fluxcalibdir = GETENV('INCAFLUX_DIR')
		datesfile = fluxcalibdir+'decom_incaflux_dates.txt'
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		OPENR,lun,datesfile,/GET_LUN,ERROR=err
		IF err LT 0 THEN BEGIN
			PRINT,'PDS_READ_IMAGECAL: Failure to open '+datesfile
			RETURN,-1
		ENDIF
		READF,lun,numstr
		datedirs = STRARR(numstr)
		dateet = DBLARR(numstr)
		buff = ''
		FOR n=0,numstr-1 DO BEGIN
			READF,lun,buff
			e=STRSPLIT(buff,' ',/EXTRACT)
			datedirs[n] = e[0]
			s=UTC2ET(e[1],et)
			dateet[n] = et
		ENDFOR	
		CLOSE,lun
		FREE_LUN,lun
		
		IF et2use GE dateet[2] THEN caldir2use = datedirs[2] $
		ELSE IF et2use GE dateet[1] tHEN caldir2use = datedirs[1] $
		ELSE caldir2use = datedirs[0]
		PRINT,'Using '+caldir2use
		
		;setup spin and stare paths, the calibration is different between them
		calspinpath = fluxcalibdir+p+caldir2use+p+'spin'+p
		calstarepath = fluxcalibdir+p+caldir2use+p+'stare'+p
		
		fstatus = DBLARR(8)
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;get image calibration file names
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		PDS_GET_IMAGE_CALFILENAMES,hispat_loTOF_neu_H_calfl,$
							hispat_loTOF_neu_O_calfl,$
							hispat_hiTOF_neu_H_calfl,$
							hispat_hiTOF_neu_O_calfl,$
							hitime_loTOF_neu_calfl,$
							hitime_hiTOF_neu_calfl,$
							hitime_loTOF_ion_calfl,$
							hitime_hiTOF_ion_calfl,$
							himTOF_neu_H_calfl,$
							himTOF_neu_O_calfl,$
							himTOF_ion_H_calfl,$
							himTOF_ion_O_calfl
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;spatial on
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		spaton = 0
		IF STRMATCH(STRLOWCASE(imagetype),'spatial') EQ 1 THEN BEGIN
			spaton = 1
			;Image matrix
			;stare files
			fstatus[0] = READMATRIX(calstarepath+hispat_loTOF_neu_H_calfl, 64, temp)
			hispat_loTOF_neu_H_caldata = TRANSPOSE(temp)
			fstatus[1] = READMATRIX(calstarepath+hispat_loTOF_neu_O_calfl, 64, temp)
			hispat_loTOF_neu_O_caldata = TRANSPOSE(temp)
			fstatus[2] = READMATRIX(calstarepath+hispat_hiTOF_neu_H_calfl, 32, temp)
			hispat_hiTOF_neu_H_caldata = TRANSPOSE(temp)
			fstatus[3] = READMATRIX(calstarepath+hispat_hiTOF_neu_O_calfl, 32, temp)
			hispat_hiTOF_neu_O_caldata = TRANSPOSE(temp)
			;spin files
			
			fstatus[4] = READMATRIX(calspinpath+hispat_loTOF_neu_H_calfl, 64, temp)
			hispat_loTOF_neu_H_caldata_spin = TRANSPOSE(temp)
			fstatus[5] = READMATRIX(calspinpath+hispat_loTOF_neu_O_calfl, 64, temp)
			hispat_loTOF_neu_O_caldata_spin = TRANSPOSE(temp)
			fstatus[6] = READMATRIX(calspinpath+hispat_hiTOF_neu_H_calfl, 32, temp)
			hispat_hiTOF_neu_H_caldata_spin = TRANSPOSE(temp)
			fstatus[7] = READMATRIX(calspinpath+hispat_hiTOF_neu_O_calfl, 32, temp)
			hispat_hiTOF_neu_O_caldata_spin = TRANSPOSE(temp)
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high spatial cal file '
				RETURN,-1
			ENDIF
			
		ENDIF	;spatial
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;time on
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		timeon = 0
		IF STRMATCH(STRLOWCASE(imagetype),'time') EQ 1 THEN BEGIN
			timeon = 1
			;Image matrix
			;stare files
			fstatus[0] = READMATRIX(calstarepath+hitime_loTOF_neu_calfl, 32, temp)
			hitime_loTOF_neu_caldata = TRANSPOSE(temp)
			fstatus[1] = READMATRIX(calstarepath+hitime_hiTOF_neu_calfl, 32, temp)
			hitime_hiTOF_neu_caldata = TRANSPOSE(temp)
			fstatus[2] = READMATRIX(calstarepath+hitime_loTOF_ion_calfl, 32, temp)
			hitime_loTOF_ion_caldata = TRANSPOSE(temp)
			fstatus[3] = READMATRIX(calstarepath+hitime_hiTOF_ion_calfl, 32, temp)
			hitime_hiTOF_ion_caldata = TRANSPOSE(temp)
			
			;spin files
			
			fstatus[4] = READMATRIX(calspinpath+hitime_loTOF_neu_calfl, 32, temp)
			hitime_loTOF_neu_caldata_spin = TRANSPOSE(temp)
			fstatus[5] = READMATRIX(calspinpath+hitime_hiTOF_neu_calfl, 32, temp)
			hitime_hiTOF_neu_caldata_spin = TRANSPOSE(temp)
			fstatus[6] = READMATRIX(calspinpath+hitime_loTOF_ion_calfl, 32, temp)
			hitime_loTOF_ion_caldata_spin = TRANSPOSE(temp)
			fstatus[7] = READMATRIX(calspinpath+hitime_hiTOF_ion_calfl, 32, temp)
			hitime_hiTOF_ion_caldata_spin = TRANSPOSE(temp)
						  
			IF MIN(fstatus[0:3]) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high time cal file '
				RETURN,-1
			ENDIF
			
		ENDIF	;time
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;tof on
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		tofon = 0
		IF STRMATCH(STRLOWCASE(imagetype),'m-tof') EQ 1  THEN BEGIN
			tofon = 1
			himTOF_neu_H_caldata = DBLARR(8,16,16)
			himTOF_neu_O_caldata = DBLARR(8,16,16)
			himTOF_ion_H_caldata = DBLARR(8,8,8)
			himTOF_ion_O_caldata = DBLARR(8,8,8)
			
			;Image matrix
			;stare files
			
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calstarepath+himTOF_neu_H_calfl[n], 16, temp)
				IF fstatus[n] EQ 0 THEN himTOF_neu_H_caldata[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Neutral H cal file '
				RETURN,-1
			ENDIF
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calstarepath+himTOF_neu_O_calfl[n], 16, temp)
				IF fstatus[n] EQ 0 THEN himTOF_neu_O_caldata[n,*,*] = TRANSPOSE(temp)
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Neutral O cal file '
				RETURN,-1
			ENDIF
			
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calstarepath+himTOF_ion_H_calfl[n], 8, temp)
				IF fstatus[n] EQ 0 THEN himTOF_ion_H_caldata[n,*,*] = TRANSPOSE(temp )
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Ion H cal file '
				RETURN,-1
			ENDIF
			
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calstarepath+himTOF_ion_O_calfl[n], 8, temp)
				IF fstatus[n] EQ 0 THEN himTOF_ion_O_caldata[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Ion O cal file '
				RETURN,-1
			ENDIF
			
			;spin files
			
			himTOF_neu_H_caldata_spin = DBLARR(8,16,16)
			himTOF_neu_O_caldata_spin = DBLARR(8,16,16)
			himTOF_ion_H_caldata_spin = DBLARR(8,8,8)
			himTOF_ion_O_caldata_spin = DBLARR(8,8,8)
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calspinpath+himTOF_neu_H_calfl[n], 16, temp)
				IF fstatus[n] EQ 0 THEN himTOF_neu_H_caldata_spin[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Neutral H cal file '
				RETURN,-1
			ENDIF
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calspinpath+himTOF_neu_O_calfl[n], 16, temp)
				IF fstatus[n] EQ 0 THEN himTOF_neu_O_caldata_spin[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Neutral O cal file '
				RETURN,-1
			ENDIF
			
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calspinpath+himTOF_ion_H_calfl[n], 8, temp)
				IF fstatus[n] EQ 0 THEN himTOF_ion_H_caldata_spin[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Ion H cal file '
				RETURN,-1
			ENDIF
			
			FOR n=0,7 DO BEGIN
				fstatus[n] = READMATRIX(calspinpath+himTOF_ion_O_calfl[n], 8, temp)
				IF fstatus[n] EQ 0 THEN himTOF_ion_O_caldata_spin[n,*,*] = TRANSPOSE(temp) 
			ENDFOR 
			IF MIN(fstatus) LT 0 THEN BEGIN
				PRINT,'PDS_READ_IMAGECAL: Failure to open high tof Ion O cal file '
				RETURN,-1
			ENDIF
			
			
		ENDIF	;tof

		calib = {hispat_loTOF_neu_H_caldata:hispat_loTOF_neu_H_caldata,$
			hispat_loTOF_neu_O_caldata:hispat_loTOF_neu_O_caldata,$
			hispat_hiTOF_neu_H_caldata:hispat_hiTOF_neu_H_caldata,$
			hispat_hiTOF_neu_O_caldata:hispat_hiTOF_neu_O_caldata,$
			hispat_loTOF_neu_H_caldata_spin:hispat_loTOF_neu_H_caldata_spin,$
			hispat_loTOF_neu_O_caldata_spin:hispat_loTOF_neu_O_caldata_spin,$
			hispat_hiTOF_neu_H_caldata_spin:hispat_hiTOF_neu_H_caldata_spin,$
			hispat_hiTOF_neu_O_caldata_spin:hispat_hiTOF_neu_O_caldata_spin,$
			hitime_loTOF_neu_caldata:hitime_loTOF_neu_caldata,$
			hitime_hiTOF_neu_caldata:hitime_hiTOF_neu_caldata,$
			hitime_loTOF_ion_caldata:hitime_loTOF_ion_caldata,$
			hitime_hiTOF_ion_caldata:hitime_hiTOF_ion_caldata,$
			hitime_loTOF_neu_caldata_spin:hitime_loTOF_neu_caldata_spin,$
			hitime_hiTOF_neu_caldata_spin:hitime_hiTOF_neu_caldata_spin,$
			hitime_loTOF_ion_caldata_spin:hitime_loTOF_ion_caldata_spin,$
			hitime_hiTOF_ion_caldata_spin:hitime_hiTOF_ion_caldata_spin,$
			himTOF_neu_H_caldata:himTOF_neu_H_caldata,$
			himTOF_neu_O_caldata:himTOF_neu_O_caldata,$
			himTOF_ion_H_caldata:himTOF_ion_H_caldata,$
			himTOF_ion_O_caldata:himTOF_ion_O_caldata,$
			himTOF_neu_H_caldata_spin:himTOF_neu_H_caldata_spin,$
			himTOF_neu_O_caldata_spin:himTOF_neu_O_caldata_spin,$
			himTOF_ion_H_caldata_spin:himTOF_ion_H_caldata_spin,$
			himTOF_ion_O_caldata_spin:himTOF_ion_O_caldata_spin}
	ENDIF	;have flux or cal matrix only

	
RETURN,0
END
