;+
; NAME:
;	PDS_READ_MAG_DATA
;
; PURPOSE:
;	To  read PDS mag data. The code attempts to read 3 types of data that is in the PDS for the Mag team.
;	It transforms from the KRTP or RTN frame into CASSINI_SC_COORD frame or the user can use FRAME
;	keyword to select the output coordinate frame for the data. This routine reads the data into 
;	a data array similar to those for the MIMI data so we can use the same indexes to find time and such.
;
; NOTE: 
;	This routine requires use of the mimi analysis spice kernel set
;	and the spice DLM to be installed for IDL. See 
;	http://cassini-mimi.jhuapl.edu/data_auxiliary.html#Kernels
;	for more information on how to get the analysis kernel set and load
;	it inside IDL.
;	See http://naif.jpl.nasa.gov/naif/toolkit.html for information on
;	getting the toolkit, look at Toolkits for IDL option.
;   
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	struct = PDS_READ_MAG_DATA(startyear, startdoy, starthours, startmins, startsecs, $
;                       stopyear, stopdoy, stophours, stopmins, stopsecs, $
;						files, all_numcols, all_numrows,all_data, labels,$
;						all_spinchar_scet, all_datachar_scet)
;
; INPUTS:
;   startyear, stopyear     4 digit integer year
;   startdoy, stopdoy       integer day of year 1-365 or 366 on leap year
;   starthours, stophour    integer hours 0-23
;   startmins, stopmins     integer minutes 0-59
;   startsecs, stopsecs     integer seconds 0-59
;	files		Array of directories and filenames for data files
;
; OUTPUTS:
;   all_numcols	        finally equals total number of columns, including header
;   all_numrows	        long total number of rows
;   all_data	        array size [numrows,numcols]
;   label	        text array of column labels, size numcols+2, includes 2 SCET
;                   following char labels at beginning
;   all_spinchar_scet   text array of spinstart spacecraft ephemeris time, size numrows
;   all_datachar_scet   text array of  data time spacecraft ephemeris time, size numrows
;
; KEYWORD PARAMETERS:
;	FRAME			As default, the Cassini MAG data will be returned in the CASSINI_SC_COORD coordinate
;					system, but the user can input any valid SPICE frame and the data will be returned
;					in that coordinate system.
;
;                       Copyright (C) 2012
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       01 Feb 2012, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_PRPM, data, labels, datascet

@pds_data_hdr_h

	file=[GETENV('HOME')+'/project/cassini/ftpdata/PDS_Volumes/COMAG_4001/DATA/KRTP/2010/10256_10277_138_FGM_KRTP_1S.TAB']
	t=['2010-260T00:00:00','2010-260T02:00:00']
	
	;file=[GETENV('HOME')+'/project/cassini/ftpdata/PDS_Volumes/COMAG_4001/DATA/KRTP/2004/04135_04240_00_FGM_KRTP_1S.TAB']
	;t=['2004-140T00:00:00','2004-140T02:00:00']
	
	file=[GETENV('HOME')+'/project/cassini/ftpdata/PDS_Volumes/COMAG_3XXX/DATA/Y2004/04002_FGM_RTN.TAB']
	t=['2004-002T00:00:00','2004-002T02:00:00']
	;file=[GETENV('HOME')+'/project/cassini/ftpdata/PDS_Volumes/COMAG_3XXX/DATA/Y2010/10001_FGM_KRTP.TAB']
	;t=['2010-001T00:00:00','2010-001T01:00:00']
	;file=[GETENV('HOME')+'/project/cassini/ftpdata/PDS_Volumes/COMAG_3XXX/DATA/Y2010/10260_FGM_KRTP.TAB']
	;t=['2010-260T00:00:00','2010-260T02:00:00']
	
	TIMESTR2NUM, t[0], startyear, startdoy, starthours, startmins, startsecs, startmsecs
	TIMESTR2NUM, t[1], stopyear, stopdoy, stophours, stopmins, stopsecs, stopmsecs
	
	
	
	s = PDS_READ_MAG_DATA(startyear, startdoy, starthours, startmins, startsecs, $
                           stopyear, stopdoy, stophours, stopmins, stopsecs, $
						   file,  numcols, numrows, data, labels, spnscet, datascet)
	
	IF s NE -1 THEN BEGIN
		nl = N_ELEMENTS(labels)

		COLUMN_WIDTH = 23; width for columns in ascii file

		cw = STRCOMPRESS(STRING(COLUMN_WIDTH),/remove_all)
		cws = STRCOMPRESS(STRING(8),/remove_all)

		nc = STRCOMPRESS(STRING(numcols),/remove_all)
		OPENW,lun,GETENV('HOME')+'/magdata.txt',/GET_LUN


		printf,lun,labels[1],labels[1],labels[nl-3:nl-1],$
			format='(a23,",",A23,", ",3(A'+cw+',", "))'
		FOR n=0,numrows-1 do begin
			year = STRMID(datascet[n],0,4)
			m = STRMID(datascet[n],5,2)
			d = STRMID(datascet[n],8,2)
			;print,'year ',year,' m ',m,' d ',d
			doy = YMD2DN(fix(year),fix(m),fix(d))
			hours = STRMID(datascet[n],10)
			printf,lun,year+'-'+NUM2STR(doy,SLONG=3)+hours,datascet[n],$
				data[n,numcols-3:numcols-1],format='(2(A23,", "),3(G'+cw+'.6,", "))'
		endfor

		close,lun
		free_lun,lun
	ENDIF
RETURN
END

FUNCTION PDS_READ_MAG_DATA, startyear, startdoy, starthours, startmins, startsecs, $
                           stopyear, stopdoy, stophours, stopmins, stopsecs, files,$
						all_numcols, all_numrows,all_data, labels,$
						all_spinchar_scet, all_datachar_scet, FRAME=FRAME

@pds_data_hdr_h

	filestat=FILE_TEST(files)
	channels=['BX','BY','BZ']

	;CATCH,err
	;IF err NE 0 THEN BEGIN
	;	PRINT,'PDS_READ_MAG_DATA: Failure to read Cassini/MAG data '
	;	PRINT,'Do you have the SPICE DLM installed?'
	;	PRINT,'Do you have the MIMI analysis kernel set installed and loaded?'
	;;	RETURN,-1
	;ENDIF
	CSPICE_KTOTAL,'ALL',cnts
	IF cnts GT 0 THEN BEGIN

    	;test inputs
    	starttimes = ['startyear','startdoy','starthours','startmins','startsecs']
    	ss = [SIZE(startyear,/TYPE),SIZE(startdoy,/TYPE),SIZE(starthours,/TYPE),$
            	SIZE(startmins,/TYPE),SIZE(startsecs,/TYPE)]
    	at = WHERE(ss NE 1 AND ss NE 2 AND ss NE 3)
    	IF at[0] NE -1 THEN BEGIN
        	PRINT,'PDS_READ_MIMI: '+starttimes[at]+' are not integers'
        	PRINT,'Exiting. Please use integers for time fields'
        	RETURN,-1
    	ENDIF

    	stoptimes = ['stopyear','stopdoy','stophours','stopmins','stopsecs']
    	ss = [SIZE(stopyear,/TYPE),SIZE(stopdoy,/TYPE),SIZE(stophours,/TYPE),$
            	SIZE(stopmins,/TYPE),SIZE(stopsecs,/TYPE)]
    	at = WHERE(ss NE 1 AND ss NE 2 AND ss NE 3)
    	IF at[0] NE -1 THEN BEGIN
        	PRINT,'PDS_READ_MIMI: '+stoptimes[at]+' are not integers'
        	PRINT,'Exiting. Please use integers for time fields'
        	RETURN,-1
    	ENDIF

		IF startyear LT 1997 OR startyear GT 2018 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in start year field ',startyear
			RETURN,-1
		ENDIF
		IF stopyear LT 1997 OR stopyear GT 2018 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in stop year field ',stopyear
			RETURN,-1
		ENDIF

		lastdoy = YMD2DN(startyear,12,31)
		IF startdoy LT 1 OR startdoy GT lastdoy THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in start day of year field ',startdoy
			RETURN,-1
		ENDIF

		lastdoy = YMD2DN(stopyear,12,31)
		IF stopdoy LT 1 OR stopdoy GT lastdoy THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in stop day of year field ',stopdoy
			RETURN,-1
		ENDIF
		IF starthours LT 0 OR starthours GT 23 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in start hours field ',starthours
			RETURN,-1
		ENDIF
		IF stophours LT 0 OR stophours GT 23 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in stop hours field ',stophours
			RETURN,-1
		ENDIF

		IF startmins LT 0 OR startmins GT 59 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in minutes field ',startmins
			RETURN,-1
		ENDIF
		IF stopmins LT 0 OR stopmins GT 59 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in minutes field ',stopmins
			RETURN,-1
		ENDIF
		IF startsecs LT 0 OR startsecs GT 59 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in seconds field ',startsecs
			RETURN,-1
		ENDIF
		IF stopsecs LT 0 OR stopsecs GT 59 THEN BEGIN
			PRINT,'PDS_READ_MIMI: Failure in seconds field ',stopsecs
			RETURN,-1
		ENDIF
		
		startutc = NUM2TIMESTR(startyear, startdoy, starthours, startmins, startsecs)
		stoputc = NUM2TIMESTR(stopyear, stopdoy, stophours, stopmins, stopsecs)
	
		fromframeKRTP = 'CASSINI_MIMI_MAG_KRTP'
		fromframeRTN = 'CASSINI_MIMI_MAG_RTN' 
		toframe = "CASSINI_SC_COORD"
		IF N_ELEMENTS(FRAME) NE 0 THEN BEGIN
			toframe = FRAME
		ENDIF
		
		print,'PDS_READ_MAG_DATA: Mag data will be returned in the '+toframe+' coordinate system'
		
		strtime = [startutc,stoputc]
		s=SPICE_UTC2ET(strtime, et)

		status = 0
    	hdrchans = ['Spinstart_SCET_char','Data_SCET_char','Spin_Counter','Sector_Number','Subsector_Number',$
		'Spinstart_SCLK_secs','Spinstart_SCLK_fine','Spinstart_ET_seconds','Cds_Error','Spinstar_Mode',$
		'Spin_Period','Data_SCLK_secs','Data_SCLK_fine','Data_ET_secs','Sensor_Bitrate']
		
		labels = [hdrchans,channels]
		all_numcols = N_ELEMENTS(labels)-2

		;NOTE bit rate is not loaded, I think I will have to load these
		;from the apl data to get my software to work.

		;figure out the number of lines in the file
		FOR n=0l,N_ELEMENTS(files)-1 DO BEGIN
			IF filestat[n] EQ 1 THEN BEGIN
				IF STRPOS(files[n],'.gz') NE -1 THEN BEGIN
					mylines = LONG(FILE_LINES(files[n],/COMPRESS))
				ENDIF ELSE BEGIN
					mylines = LONG(FILE_LINES(files[n]))
				ENDELSE
				nlines = (N_ELEMENTS(nlines) EQ 0) ? mylines : nlines+mylines

			ENDIF
		ENDFOR
		IF N_ELEMENTS(nlines) EQ 0 THEN BEGIN
			PRINT,'PDS_READ_MAG_DATA: Failure to locate file '+files
			RETURN,-1
		ENDIF
		all_numcols = N_ELEMENTS(labels)-2
		all_data = DBLARR(nlines,all_numcols)
		all_spinchar_scet=STRARR(nlines)
		all_datachar_scet=STRARR(nlines)

		hour = 60*60

		nl = 0l
		buff = ''
		mysclk = ULONG(0)	;must save here outside of file loop so can use from previous file
		myfine = 0L
		FOR n=0l,N_ELEMENTS(files)-1 DO BEGIN
			krtppos = STRPOS(files[n],'KRTP')
			sec1pos = STRPOS(files[n],'_1S')
			fromframe = (krtppos[0] NE -1) ? fromframeKRTP : fromframeRTN
								print,'sce1pos ',sec1pos
			IF filestat[n] EQ 1 THEN BEGIN
				print,'Reading '+files[n]+'...'
				IF STRPOS(files[n],'.gz') NE -1 THEN BEGIN
					OPENR,lun,files[n],/GET_LUN,ERROR=err,/COMPRESS
				ENDIF ELSE BEGIN
					OPENR,lun,files[n],/GET_LUN,ERROR=err
				ENDELSE
				IF err LT 0 THEN BEGIN
					PRINT,'Failure to read from '+files[n]
				ENDIF ELSE BEGIN
					done = 0
					tov = DBLARR(3)
					WHILE NOT EOF(lun) AND done NE 1 DO BEGIN
						READF,lun,buff

						e=STRSPLIT(buff,' ',/EXTRACT)

						IF N_ELEMENTS(e) GE 4 THEN BEGIN
							de = [DOUBLE(e[1:*])]
							s=SPICE_SCETSCLK(e[0], usclk, fine, myet)

							;if file is multi-day 1 second average file, then skip through until 
							;within 1 hour
							IF sec1pos[0] NE -1 AND myet LT et[0]-hour THEN BEGIN
								FOR p=0,59 DO READF,lun,buff
								e=STRSPLIT(buff,' ',/EXTRACT)
								s=SPICE_SCETSCLK(e[0], usclk, fine, myet)
							ENDIF


							;the data is read into an array dimentioned like the MIMI data just
							;so we can use the same indexes to find time and such.
							IF myet GE et[0] AND myet LE et[1] THEN BEGIN

								;load header data
								all_datachar_scet[nl] = e[0]

								all_data[nl,SPIN_COUNTER_C] = 256	;fill value, not used
								all_data[nl,SECTOR_NUMBER_C] = 256	;fill value, not used
								all_data[nl,SUBSECTOR_NUMBER_C] = 256	;fill value, not used
								all_data[nl,SPINSTAR_MODE_C] = 256	;fill value, not used
								all_data[nl,SPIN_PERIOD_C_C] = 256	;fill value, not used
								all_data[nl,DATA_SCLK_SECS_C] = usclk
								all_data[nl,DATA_ET_SECS_C] = myet	;fill value, not used
								all_data[nl,SENSOR_BITRATE_C] = 256

								;convert from appropriate frame to SC_COORD Frame
								vector_from = [de[0],de[1],de[2]]

								vector_to = PDS_XFORM_VECTOR(fromframe, myet, toframe, myet, vector_from, $
									status=status)

								all_data[nl,SENSOR_BITRATE_C+1] =  vector_to[0]
								all_data[nl,SENSOR_BITRATE_C+2] = vector_to[1]
								all_data[nl,SENSOR_BITRATE_C+3] = vector_to[2]

								nl++
							ENDIF
							IF myet GT et[1] THEN done = 1
						ENDIF
					ENDWHILE

					CLOSE,lun
					FREE_LUN,lun
				ENDELSE
			ENDIF
		ENDFOR

		IF nl GT 0 THEN BEGIN
			all_numrows = nl
			all_data = all_data[0:all_numrows-1,*]
			all_spinchar_scet = all_spinchar_scet[0:all_numrows-1]
			all_datachar_scet = all_datachar_scet[0:all_numrows-1]
		ENDIF ELSE BEGIN
			all_numrows = 0
			PRINT,'READ_PDS_MAG:Failure to find matching data'
			status = -1
		ENDELSE

	ENDIF ELSE BEGIN
		PRINT,'PDS_READ_MAG_DATA: Failure to read Cassini MAG data '
		PRINT,'Do you have the MIMI analysis kernel set installed and loaded?'
		RETURN,-1
	ENDELSE
	
	RETURN,status
END
