;+
; NAME:
;	PDS_READ_MIMI
;
; PURPOSE:
;	To  read MIMI non-image PDS data. All files passed in MUST be from the same data type.
;   
; NOTE: This routine uses ET2UTC and UTC2ET to convert between ET ephemeris time
;	which is the time base used in the file and UTC a string time representation.
;	These routines should be used if the user does not have the JPL NAIF SPICE 
;	software installed. 
;
;	But for more accuracy,  the user can install the SPICE IDL DLM and obtain 
;	the leap seconds and sclk kernels, they can switch to using SPICE_ET2UTC and 
;	SPICE_UTC2ET. Just replace the calls in this routine with those. The user must 
;	load the kernels before using the spice library.
;	CSPICE_FURNSH,<path>/cas00147.tsc
;	CSPICE_FURNSH,<path>/naif0009.tls
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	status = PDS_READ_MIMI(startyear, startdoy, starthours, startmins, startsecs, $
;                           stopyear, stopdoy, stophours, stopmins, stopsecs, files, $
;                           labels, data, auxdata, auxlabels)
;
; INPUTS:
;   startyear, stopyear     4 digit integer year
;   startdoy, stopdoy       integer day of year 1-365 or 366 on leap year
;   starthours, stophour    integer hours 0-23
;   startmins, stopmins     integer minutes 0-59
;   startsecs, stopsecs     integer seconds 0-59
;   sfile                    Input path and PDS MIMI filenames with extention
;
; OUTPUTS:
;   labels                  String array containing data column labels [numcols]
;   data                    Double array containing data [numrows, numcols]
;	utc						String arraay containing the utc of the data [numrows]
;   auxlabels         String array containing auxiliary data column labels [6]
;   auxdata           Double array containing auxiliary data [6, numcols]
;
;   RETURNS 0 indicates that the file was found and data was returned
;           -1 indicates that either the file was not found or was unable to read or no data matched.
;
; KEYWORD PARAMETERS:
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       01 Oct 2010, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;       10 Nov 2011, M.B. Kusterer Updated for change in fields for PDS data.
;       23 Mar 2012, M.B. Kusterer Added test for capitalized GZ as well as gz.
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FUNCTION PDS_READ_MIMI,startyear, startdoy, starthours, startmins, startsecs, $
                           stopyear, stopdoy, stophours, stopmins, stopsecs, files, $
                           labels, data, datautc, auxdata, auxlabels
	
@pds_data_hdr_h     ;include data header column indexes

    ;test inputs
    starttimes = ['startyear','startdoy','starthours','startmins','startsecs']
    ss = [SIZE(startyear,/TYPE),SIZE(startdoy,/TYPE),SIZE(starthours,/TYPE),$
            SIZE(startmins,/TYPE),SIZE(startsecs,/TYPE)]
    at = WHERE(ss NE 1 AND ss NE 2 AND ss NE 3)
    IF at[0] NE -1 THEN BEGIN
        PRINT,'PDS_READ_MIMI: '+starttimes[at]+' are not integers'
        PRINT,'Exiting. Please use integers for time fields'
        RETURN,-1
    ENDIF
    
    stoptimes = ['stopyear','stopdoy','stophours','stopmins','stopsecs']
    ss = [SIZE(stopyear,/TYPE),SIZE(stopdoy,/TYPE),SIZE(stophours,/TYPE),$
            SIZE(stopmins,/TYPE),SIZE(stopsecs,/TYPE)]
    at = WHERE(ss NE 1 AND ss NE 2 AND ss NE 3)
    IF at[0] NE -1 THEN BEGIN
        PRINT,'PDS_READ_MIMI: '+stoptimes[at]+' are not integers'
        PRINT,'Exiting. Please use integers for time fields'
        RETURN,-1
    ENDIF
    
	IF startyear LT 1997 OR startyear GT 2018 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in start year field ',startyear
		RETURN,-1
	ENDIF
	IF stopyear LT 1997 OR stopyear GT 2018 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in stop year field ',stopyear
		RETURN,-1
	ENDIF

	lastdoy = YMD2DN(startyear,12,31)
	IF startdoy LT 1 OR startdoy GT lastdoy THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in start day of year field ',startdoy
		RETURN,-1
	ENDIF
	
	lastdoy = YMD2DN(stopyear,12,31)
	IF stopdoy LT 1 OR stopdoy GT lastdoy THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in stop day of year field ',stopdoy
		RETURN,-1
	ENDIF
	IF starthours LT 0 OR starthours GT 23 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in start hours field ',starthours
		RETURN,-1
	ENDIF
	IF stophours LT 0 OR stophours GT 23 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in stop hours field ',stophours
		RETURN,-1
	ENDIF
	
	IF startmins LT 0 OR startmins GT 59 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in minutes field ',startmins
		RETURN,-1
	ENDIF
	IF stopmins LT 0 OR stopmins GT 59 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in minutes field ',stopmins
		RETURN,-1
	ENDIF
	IF startsecs LT 0 OR startsecs GT 59 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in seconds field ',startsecs
		RETURN,-1
	ENDIF
	IF stopsecs LT 0 OR stopsecs GT 59 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure in seconds field ',stopsecs
		RETURN,-1
	ENDIF
	
	startutc = NUM2TIMESTR(startyear, startdoy, starthours, startmins, startsecs)
	stoputc = NUM2TIMESTR(stopyear, stopdoy, stophours, stopmins, stopsecs)
	
	s=UTC2ET(startutc, startet)
	s=UTC2ET(stoputc, stopet)
	
	IF stopet LT startet THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure stoptime '+stoputc+' is less than starttime '+startutc
		RETURN,-1
	ENDIF
	
	
	;test files for the same data type
	filestubs = STRMID(FILE_BASENAME(files),0,5)
	nfiles = N_ELEMENTS(files)
	FOR n=0,nfiles-1 DO BEGIN
		IF STRMATCH(filestubs[n],filestubs[0]) NE 1 THEN BEGIN
			PRINT,'PDS_READ_MIMI: All files MUST have the same datatype!'
			RETURN,-1
		ENDIF
		IF STRMATCH(STRMID(filestubs[n],0,4),'IIMG') EQ 1 THEN BEGIN
			PRINT,'PDS_READ_MIMI: This routine does not read MIMI INCA Images!'
			PRINT,'Please use PDS_READ_MIMIIMG instead'
			RETURN,-1
		ENDIF
	ENDFOR
	
	;test files for existence and readability
	filestat = LONARR(nfiles)
	FOR n=0l,nfiles-1 DO BEGIN
		filestat[n] = FILE_TEST(files[n],/READ)
		;check if it is gzipped or unzipped, if so then try the other 
		IF filestat[n] EQ 0 THEN BEGIN
			pos = STRPOS(files[n],'.',/REVERSE_SEARCH)
			IF pos[0] NE -1 THEN BEGIN
				ext = STRMID(files[n],pos)
				IF ext EQ '.gz' THEN BEGIN
					files[n] = STRMID(files[n],0,pos[0])
					filestat[n] = FILE_TEST(files[n],/READ)
				ENDIF ELSE  BEGIN
					files[n] = files[n]+'.gz'
					filestat[n] = FILE_TEST(files[n],/READ)
				ENDELSE
			ENDIF
		ENDIF
		IF filestat[n] EQ 0 THEN PRINT,'PDS_READ_MIMI: Failure to find/read '+files[n]
	ENDFOR
	
	IF MAX(filestat) NE 1 THEN BEGIN
		PRINT,'PDS_READ_MIMI: Failure to find files'
		RETURN,-1
	ENDIF
	
	;figure out the number of lines in the file
	FOR n=0l,nfiles-1 DO BEGIN
		IF filestat[n] EQ 1 THEN BEGIN
			IF STRPOS(files[n],'.gz') NE -1 THEN BEGIN
				mylines = LONG(FILE_LINES(files[n],/COMPRESS))
			ENDIF ELSE BEGIN
				mylines = LONG(FILE_LINES(files[n]))
			ENDELSE
			nlines = (N_ELEMENTS(nlines) EQ 0) ? mylines : nlines+mylines
			
		ENDIF
	ENDFOR
	
	setuparr = 0
	nl = 0l
	buff = ''
	mysclk = ULONG(0)	;must save here outside of file loop so can use from previous file
	myfine = 0L
	FOR n=0,N_ELEMENTS(files)-1 DO BEGIN
		IF filestat[n] EQ 1 THEN BEGIN
		
			IF STRPOS(files[n],'.gz') NE -1 THEN BEGIN
				OPENR,lun,files[n],/GET_LUN,ERROR=err,/COMPRESS
			ENDIF ELSE BEGIN
				OPENR,lun,files[n],/GET_LUN,ERROR=err
			ENDELSE
			IF err LT 0 THEN BEGIN
				PRINT,'PDS_READ_MIMI: Failure to read from '+files[n]
			ENDIF ELSE BEGIN

				;get column titles and remove " from names
				READF,lun,buff
				labels = STRSPLIT(buff,',',/EXTRACT,/PRESERVE_NULL)
				numcols = N_ELEMENTS(labels)
				FOR c=0,numcols-1 DO BEGIN
					labels[c] = STRMID(labels[c],1,strlen(labels[c])-2)
				ENDFOR
                ;take off purpose and UTC from label set
				labels = labels[2:*] 
				numcols = N_ELEMENTS(labels)
                
				
				;for first file make arrays and read in auxiliary data
				IF setuparr EQ 0 THEN BEGIN
					data = DBLARR(nlines, numcols)
					datautc = STRARR(nlines)
					auxdata = DBLARR(6, numcols)
					auxlabels = STRARR(6)

					;read validmin, max, min,max,mean,stdev
					FOR c=0,5 DO BEGIN
						READF,lun,buff	
						e = STRSPLIT(buff,',',/EXTRACT,/PRESERVE_NULL)
						auxdata[c,0] = 0d
						auxdata[c,1] = DOUBLE(e[3])
						auxdata[c,2] = DOUBLE(e[4])
						auxdata[c,3:*] = DOUBLE([e[5:*]])
						auxlabels[c] = STRMID(e[0],1,strlen(e[0])-2)
					ENDFOR
					setuparr = 1
					
				;for 2nd and any following files, disgard auxiliary data
				ENDIF ELSE  BEGIN
					;read validmin,max, min,max,mean,stdev
					FOR c=0,5 DO READF,lun,buff	
				ENDELSE
				
				done = 0
				WHILE NOT EOF(lun) AND done NE 1 DO BEGIN
					READF,lun,buff
					e=STRSPLIT(buff,',',/EXTRACT,/PRESERVE_NULL)
					
					IF N_ELEMENTS(e) GE 13 THEN BEGIN
					
						;check for empty fields marked with "" 
						at = STRMATCH(e,'""')
						pos = WHERE(at EQ 1)
						IF pos[0] NE -1 THEN e[pos] = '0'
                        
                        ;extract partition 1/ from SCLK
						sclk = STRMID(e[2],3,(STRLEN(e[2])-4))
						de = DOUBLE([sclk,e[3:*]])

						IF de[DATA_ET_SECS_C] GE startet AND de[DATA_ET_SECS_C] LE stopet THEN BEGIN
							;load header data
							datautc[nl] = e[1]

							data[nl,*] = de
							nl++
						ENDIF 
						IF de[1] GT stopet+3600. THEN done = 1
					ENDIF
				ENDWHILE

				CLOSE,lun
				FREE_LUN,lun
			ENDELSE
		ENDIF
	ENDFOR
	
	IF nl GT 0 THEN BEGIN
		data = data[0:nl-1,*]
		datautc = datautc[0:nl-1]
		status = 0
	ENDIF ELSE BEGIN
		all_numrows = 0
		PRINT,'PDS_READ_MIMI:Failure to find matching data'
		status = -1
	ENDELSE
	
	

return,status
END
