;+
; NAME:
;	READ_LSK
;
; PURPOSE:
;	Read in leap kernel information and store in a common block. Requires that the
;   environmental LEAPKERNEL_FILE is set (done in setup.pro)
;
;   The leap second file is obtained from "ftp://naif.jpl.nasa.gov/pub/naif/CASSINI/kernels/lsk/" 
;   and the user should check in January and August of each year to see if any leap seconds have 
;   been added.
;   
;
; CATEGORY:
;	Support File. 
;
; CALLING SEQUENCE:
;	s=READ_LSK()
;
; INPUTS:
;
; OUTPUTS:
;   stores ls structure in common LEAP

; LS STRUCTURE:
;   The DELTA_AT in the leap second kernel are the leap seconds which represent the 
;   difference between TAI (international atomic time) and UTC (Coordinated Universal Time).
;   The differences are listed with the times at which they are active.
;
;   diff    The number of leap seconds 
;   date    The date at which the leap seconds are active
;   jsecs   The julian seconds equivalent to the date.
;   tai     The TAI equivalent to the date.
;	delta_at = {diff:long(da[0]), date:utc, jsecs:jsecs, tai:jsecs+double(da[0])}
;
;   deltaTa  = DOUBLE() DELTA_T_A in the leap second kernel. It is the difference between
;               TDT - TAI. TDT is Terrestrial dynamical time  but we are assuming TDT = TDB
;   k        = DOUBLE() DELTET/K in the leap second kernel, a constant used in ET - TAI
;   eb       = DOUBLE() DELTET/EB in the leap second kernel, a constant used in ET - TAI,
;               E is the eccentric anomaly of the heliocentric orbit of the Earth-Moon barycenter.
;   m        = DBLARR(2) DELTET/M in the leap second kernel where M is the mean anomaly, 
;               which in turn is given by M = M0  +  M1 t where t is the number of ephemeris 
;               seconds past J2000.
;
;	ls       = {deltaTa:deltaTa, k:k, eb:eb, m:m, delta_at:delta_at}
;
;;
;   RETURNS 0 return will always be 0, left in to make it easier to replace
;		with spice routines.
;           
; KEYWORD PARAMETERS:
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       11 May 2011, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_RL

COMMON LEAP, ls
 
	s=READ_LSK()
	IF s NE -1 THEN BEGIN
		ntags = N_TAGS(ls)
		tags = TAG_NAMES(ls)
		FOR n=0,ntags-1 do begin
			print,tags[n],' ',ls.(n)
		ENDFOR
	ENDIF

RETURN
END

FUNCTION CHNGDATE, date
    strmons = ['','jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec']
    date = STRMID(date,1)
    mon = STRLOWCASE(STRMID(date,5,3))
    imon = WHERE(strmons EQ mon)
    p=STRPOS(date,'-',/REVERSE_SEARCH)
    day = STRMID(date,p+1)
    doy = YMD2DN(long(STRMID(date, 0, 4)),imon,day)
    utc = STRMID(date, 0, 4)+'-'+NUM2STR(doy,SLONG=3)+'T00:00:00.000'
    
RETURN,utc
END

FUNCTION READ_LSK

COMMON LEAP, ls

    ;check that filename for leap second is ok
    filename = getenv('LEAPKERNEL_FILE')
    IF  FILE_TEST(filename) NE 1 THEN BEGIN
        SETUP
        IF  SIZE(ls,/TYPE) NE 8  THEN BEGIN
            PRINT,'READ_LSK: Error, please set LEAPKERNEL_FILE envirionmental variable in setup.pro and run setup'
            RETURN,-1
        ENDIF
    ENDIF

	data = 0
	
	OPENR,lun,filename,/GET_LUN,ERR=err
	IF err LT 0 THEN BEGIN
		PRINT,'READ_LSK: Failure to open '+filename
		RETURN,-1
	ENDIF
	
	;make sure it is the correct kernel
	buff=''
	READF,lun,buff
	IF STRPOS(buff,'KPL/LSK') EQ -1  THEN BEGIN
		PRINT,'READ_LSK: File '+filename+' is not a Leap seconds kernel file'
		RETURN,-1
	ENDIF
		
	
	;read until find "\begindata"
	done = 0
	WHILE (done NE 1) DO BEGIN
		READF,lun,buff
		IF STRPOS(buff,'\begindata') NE -1 THEN done = 1
	ENDWHILE
	
	;now read in data
	done = 0
	WHILE (done NE 1) DO BEGIN
		READF,lun,buff
		de = STRSPLIT(buff,'=',/EXTRACT)
		IF STRPOS(buff,'DELTA_AT') NE -1 THEN BEGIN
			p=STRPOS(buff,'(')
			buff = STRMID(buff, p+1)
			da = STRCOMPRESS(STRSPLIT(buff,',',/EXTRACT),/REMOVE_ALL)
            utc = CHNGDATE(da[1])
            jsecs = UTC2JSECS(utc)
			delta_at = {diff:long(da[0]), date:utc, jsecs:jsecs, tai:jsecs+double(da[0])}
			done1=0
			WHILE (done1 NE 1) DO BEGIN
				READF,lun,buff
				IF STRPOS(buff,')') NE -1 THEN BEGIN
					done1 = 1
					p=STRPOS(buff,')')
					buff = STRMID(buff, 0,p-1)
				ENDIF
				da = STRCOMPRESS(STRSPLIT(buff,',',/EXTRACT),/REMOVE_ALL)
                utc = CHNGDATE(da[1])
                jsecs = UTC2JSECS(utc)
				delta_at = [delta_at,{diff:long(da[0]), date:utc, jsecs:jsecs, $
                    tai:jsecs+double(da[0])}]
			ENDWHILE
		ENDIF
		IF N_ELEMENTS(de) GE 2 THEN BEGIN
			pb = STRPOS(de[1],'(')
			pe = STRPOS(de[1],')')
			IF pb[0] NE -1 THEN de[1] = STRMID(de[1],pb+1)
			IF pe[0] NE -1 THEN de[1] = STRMID(de[1],0,pe-2)
			IF STRPOS(buff,'DELTA_T_A') NE -1 THEN deltaTa = DOUBLE(de[1])
			IF STRPOS(buff,'DELTET/K') NE -1 THEN k = DOUBLE(de[1])
			IF STRPOS(buff,'DELTET/EB') NE -1 THEN eb = DOUBLE(de[1])
			IF STRPOS(buff,'DELTET/M') NE -1 THEN m = DOUBLE(STRSPLIT(de[1],' ',/EXTRACT))

		ENDIF
			
		IF STRPOS(buff,'\begintext') NE -1 THEN done = 1
	ENDWHILE
	
	
	ls = {deltaTa:deltaTa, k:k, eb:eb, m:m, delta_at:delta_at}
	
	CLOSE,lun
	FREE_LUN,lun
RETURN,0
END
