;+
; NAME:
;	TEST_GET_INCA_ATT
;
; PURPOSE:
;	Read in INCA data to allow user to test routine GET_INCA_ATT which
;	obtains pointing of INCA pixels in INCA FOV frame 'CASSINI_MIMI_INCA_LL'
;	corrected by the offsets of the transmitted images if appropriate. It also
;	calls PDS_GET_PITCH_ANGLE which returns the pitch angle for each image
;	pixel. It calls PDS_READ_MAG_DATA which reads in the Cassini MAG team
;	data. 
;
; NOTE: 
;	This routine requires use of the mimi analysis spice kernel set
;	and the spice DLM to be installed for IDL. See 
;	http://cassini-mimi.jhuapl.edu/data_auxiliary.html#Kernels
;	for more information on how to get the analysis kernel set and load
;	it inside IDL.
;	See http://naif.jpl.nasa.gov/naif/toolkit.html for information on
;	getting the toolkit, look at Toolkits for IDL option.
;   
; NOTE: 
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	TEST_GET_INCA_ATT, att
;
; INPUTS:
;	useet			Ephemeris time, use UTC2ET to convert from yyyy-doyThh:mm:ss.msc
;					(see example below)
;
; KEYWORD PARAMETERS:
;	USEET			Array of ET ephemeris times to find pointing at.
;
;
;
;                       Copyright (C) 2012
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       03 August 2012, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_GET_INCA_ATT, att

@pds_data_hdr_h        ;include data header column indexes
@pds_image_hdr_h     ;include inca image header column indexes
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;  NOTE:
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(*) User Action: in this routine there are eight locations the user  
	;      needs to edit variables in order to list their own selected files 
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(1) set up data location, yours can be just a simple topdir='/user/frank/'
	;as it is, directory would be
	;/project/cassini/df004/PDS_Volumes/COMIMI_0SAT/DATA/<sensordir>/<year>/filename
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	imagepds = 'INCA_IMAGES'
	p=PATH_SEP()	;is "/" for unix, linux, darwin
	topdir = (!version.os EQ 'darwin') ? $
		GETENV('HOME')+p+'project'+p+'cassini'+p+'ftpdata'+p+'PDS_Volumes'+p+'COMIMI_0SAT'+p+'DATA'+p : $
		p+'project'+p+'cassini'+p+'df004'+p+'PDS_Volumes'+p+'COMIMI_0SAT'+p+'DATA'+p
		

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
	; Table of index values versus species, tof, charge and imagetype, see below after the
	;	images are read, the data array is used to search for specific images using
	;	time, type, charge, species and tof
	;
	; The file pds_image_hdr_h.pro contains the following indexes which are set to locate
	;	the fields in the data array that correspond to the images. See an 
	;	example in the WHERE call which takes place after the read of the images.
	;
	; TYPE_C 	0=High Spatial Resolution
	;			1=High Time Resolution
	;			2=High mTOF Resolution
	; CHARGE_C  0=Image is of neutrals
	;			1=Image is of charged particles and neutrals 
	; SPECIE_C 	0=Hydrogen
	;			1=Not used
	;			2=Oxygen
	;			3=Not used
	;			4=Not used
	;			5=All Species (only for high time resolution images)
	; TOF_C 	0=Low Tof
	;			7=High Tof for high spatial and high time resolution images
	;			0,1,2,3,4,5,6,7 for high mTOF resolution images
	; NUMROWS_C and NUMCOLS_C are the number of rows and columns in the images,
	;			they are always the same.
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(2) set up image type, the read routine takes
	;only specific strings, after the data is read, 
	;the mytype will be used to locate high  
	; m-TOF resolution images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	imagetypes = ['Spatial','Time','m-TOF']	
	mytype = 2
		
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(3) set up units
	;0 = Counts
	;1 = Counts/s
	;2 = Counts/(cm*s*sr)
	;3 = Counts/(cm*s*sr*Kev)
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	unittypes = ['Counts','Counts/s', 'Counts/(cm*s*sr)','Counts/(cm*s*sr*Kev)']
	myunit = 3
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(4) Species, after the data is read, the myspecies variable
	; will be used to locate H images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	species = ['H','NotUsed','O','NotUsed','NotUsed','All']
	myspecies = 0
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(5) if using high m-tof images, the tof = 3 
	; selects the energy range is 55-90 keV  , 
	; after the data is read, the myspecies variable
	; will be used to locate 55-90 keV images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	mytof = 3

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(6) set up titles for charge, after the data is read, the mycharge variable
	; will be used to locate Ion images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	charge = ['Neutral','Ion']
	mycharge = 0
		
	startyear = 2010
	startdoy = 267
	starthours = 18
	startmins = 0
	startsecs = 0
	
	stopyear = 2010
	stopdoy = 267
	stophours = 18
	stopmins = 05
	stopsecs = 22
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;convert times to ascii UTC string to ephemeris time, 
	;et is dblarr[2], 0=starting time, 1=ending time
	;ephemeris time makes it easier to find matching times
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	strtime = NUM2TIMESTR(startyear, startdoy, starthours, startmins, startsecs)
	stptime = NUM2TIMESTR(stopyear, stopdoy, stophours, stopmins, stopsecs)
	s=UTC2ET([strtime,stptime],et)
	

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;Read in MIMI INCA Ion/Neutral Time range files
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	status = READ_NEUIONMODES(strtime,stptime, neuionmodes)
	
	
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(7) set up INCA image and accumulator rates filenames
	; call read routine, only the image type selected will be returned, but each image 
	; type contains multiple TOF, charge and species images!
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	files = [topdir+'INCA_IMAGES'+p+'2010'+p+'IIMG0_2010267_0003.CSV']
	arfiles = [topdir+'INCA_ACCUMULATION_RATES'+p+'2010'+p+'IACC0_2010267_0003.CSV']
	s = PDS_READ_MIMIIMG(startyear, startdoy, starthours, startmins, startsecs, $
                    stopyear, stopdoy, stophours, stopmins, stopsecs,  $
					files, arfiles, imagetypes[mytype], unittypes[myunit], $
                    labels, data, images, datautc, stopdatautc,$
					auxdata, auxlabels, bandwidths, energy_ranges)
					
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(8) set up MAG filename 
	;call Cassini MAG read routine
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	mfile=[GETENV('HOME')+'/project/cassini/df004/PDS_Volumes/COMAG_4001/DATA/KRTP/2010/10256_10277_138_FGM_KRTP_1S.TAB']
	ms = PDS_READ_MAG_DATA(startyear, startdoy, starthours, startmins, startsecs, $
                           stopyear, stopdoy, stophours, stopmins, stopsecs, $
						   mfile, mnumcols, mnumrows, mdata, mlabels, mspnscet, mdatascet)
						   
						   
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(9) set up output filenames for INCA image attitude and pitch angle
	; Enter the frame to calculate the image pixel attitude in
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	attfile = getenv('HOME')+'/attitude.txt'
	pitchfile = getenv('HOME')+'/pitchangle.txt'
	frame = 'XINCA_BORESIGHT_CENTERED'

	IF s EQ 0 THEN BEGIN
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;Load an array the same size as the images data array with the matching INCA 
		;neutral/ion modes
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		modes = LONARR(N_ELEMENTS(data[*,DATA_ET_SECS_C])+5)
		mymode = 0l
		
		FOR n=0,N_ELEMENTS(neuionmodes.mode)-1 DO BEGIN
			s=UTC2ET(neuionmodes.utc[*,n], ni_ets)
			
			mymode = neuionmodes.mode[n]
			at = WHERE(data[*,DATA_ET_SECS_C] GE ni_ets[0] AND $
				data[*,DATA_ET_SECS_C] LE ni_ets[1])
			
			IF at[0] NE -1 THEN BEGIN
				modes[at] = neuionmodes.mode[n]
			ENDIF
		ENDFOR
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		; search for image time, type, tof and species. Only images with 
		; type = high m-TOF will be returned but the test is left for completness.
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		at = WHERE((data[*,DATA_ET_SECS_C] GE et[0] AND $
				data[*,DATA_ET_SECS_C] LE et[1]) AND $
				(data[*,TYPE_C] EQ mytype AND $			
				data[*,SPECIE_C] EQ myspecies AND $		
				modes[*] EQ mycharge AND $		
				data[*,TOF_C] EQ mytof))
				
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
		;call routine to get inca attitude
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;				
		IF at[0] NE -1 THEN BEGIN
		
			s=GET_INCA_ATT(data[at,DATA_ET_SECS_C],data[at,NUMROWS_C],$
				data[at,TYPE_C],data[at,PHIOFFSET_C], data[at,THETAOFFSET_C], $
				data[at,SPINSTAR_MODE_C],frame,att)
			
			;just for first image, print out lat and lon for pixel attitude	
			OPENW,lun,attfile,/get_lun
			nrows = data[0,NUMROWS_C]
			snrows = NUM2STR(LONG(nrows))
			
			s=et2utc(data[at[0],DATA_ET_SECS_C],utc)
			printf,lun,utc
			printf,lun,'LAT'
			FOR y=0,nrows-1 DO BEGIN
				printf,lun,att[0,*,y].lat,format='('+snrows+'(f10.2,", "))'
			ENDFOR
			printf,lun,'LON'
			FOR y=0,nrows-1 DO BEGIN
				printf,lun,att[0,*,y].lon,format='('+snrows+'(f10.2,", "))'
			ENDFOR
			CLOSE,lun
			FREE_LUN,lun

			;calculate the pitch angle for INCA 
			IF ms NE -1 THEN BEGIN
				;set mag tolerance, we generally use 
				magtol = 30 ;degrees
				
				spa = GET_INCA_PITCH(data[at,DATA_ET_SECS_C],data[at,END_DATA_ET_SECS_C],$
					 mdata, data[at,TYPE_C], data[at,CHARGE_C], data[at,SPECIE_C], magtol, $
                    data[at,NUMROWS_C], data[at,PHIOFFSET_C], data[at,THETAOFFSET_C], data[at,SPINSTAR_MODE_C], angle)
					
				IF spa NE -1 THEN BEGIN
					;just for first image, print out lat and lon for pixel pitch angle	
					OPENW,lun,pitchfile,/get_lun
					nrows = data[0,NUMROWS_C]
					snrows = NUM2STR(LONG(nrows))

					s=et2utc([data[at[0],DATA_ET_SECS_C],data[at[0],END_DATA_ET_SECS_C]],utc)
					printf,lun,utc
					FOR y=0,nrows-1 DO BEGIN
						printf,lun,angle[0,*,y],format='('+snrows+'(f10.2,", "))'
					ENDFOR
					CLOSE,lun
					FREE_LUN,lun
				ENDIF
			ENDIF
		ENDIF
		
	ENDIF
	
RETURN
END
