;+
; NAME:
;	TEST_READ_MIMIIMG
;
; PURPOSE:
;	Example routine which calls PDS_READ_MIMIIMG to read MIMI image PDS data. 
;   Please take a look at the header of PDS_READ_MIMIIMG for more information on the routine.
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;   TEST_READ_MIMIIMG, data, images, labels, datautc, auxdata, auxlabels
;
; INPUTS:
;
; OUTPUTS:
;   labels                  String array containing data column labels [numcols]
;   data                    Double array containing data [numrows, numcols]
;   images                  Double array containing data [numrows, maximagesize, maximagesize]
;	utc						String arraay containing the utc of the data [numrows]
;   auxdata           Double array containing auxiliary data [6, numcols]
;   auxlabels         String array containing auxiliary data column labels [6]
;   bandwidth           	Double array containing image energy bandwidth [numrows]
;   energyrange           	Double array containing image energy range [2, numrows]
;
;
; KEYWORD PARAMETERS:
;
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       10 Nov 2011, M.B. Kusterer 
;
; MODIFICATION HISTORY:
;       02 Dec 2011, M.B. Kusterer Moved transpose of images into pds_read_mimiimg
;						instead of this routine.
;       07 Dec 2011, M.B. Kusterer  Added pds_load_color_indexes to control the 
;		color indexes for the axis and titles.
;       23 Mar 2012, M.B. Kusterer Combined time definitions together to be less confusing. 
;		Added more error testing.
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO TEST_READ_MIMIIMG, data, images, labels, datautc, auxdata, auxlabels, $
	MAKE_PNG=MAKE_PNG

@pds_image_hdr_h   ; include these files to get predefined indexes into image data array
@pds_data_hdr_h 


	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;  NOTE:
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(*) User Action: in this routine there are eight locations the user  
	;      needs to edit variables in order to list their own selected files 
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(1) set up data location, yours can be just a simple topdir='/user/frank/'
	;as it is, directory would be
	;/project/cassini/df004/PDS_Volumes/COMIMI_0SAT/DATA/<sensordir>/<year>/filename
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	imagepds = 'INCA_IMAGES'
	p=PATH_SEP()	;is "/" for unix, linux, darwin
	topdir = (!version.os EQ 'darwin') ? $
		GETENV('HOME')+p+'project'+p+'cassini'+p+'ftpdata'+p+'PDS_Volumes'+p+'COMIMI_0SAT'+p+'DATA'+p : $
		p+'project'+p+'cassini'+p+'df004'+p+'PDS_Volumes'+p+'COMIMI_0SAT'+p+'DATA'+p
		

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
	; Table of index values versus species, tof, charge and imagetype, see below after the
	;	images are read, the data array is used to search for specific images using
	;	time, type, charge, species and tof
	;
	; The file pds_image_hdr_h.pro contains the following indexes which are set to locate
	;	the fields in the data array that correspond to the images. See an 
	;	example in the WHERE call which takes place after the read of the images.
	;
	; TYPE_C 	0=High Spatial Resolution
	;			1=High Time Resolution
	;			2=High mTOF Resolution
	; CHARGE_C  0=Image is of neutrals
	;			1=Image is of charged particles and neutrals 
	; SPECIE_C 	0=Hydrogen
	;			1=Not used
	;			2=Oxygen
	;			3=Not used
	;			4=Not used
	;			5=All Species (only for high time resolution images)
	; TOF_C 	0=Low Tof
	;			7=High Tof for high spatial and high time resolution images
	;			0,1,2,3,4,5,6,7 for high mTOF resolution images
	; NUMROWS_C and NUMCOLS_C are the number of rows and columns in the images,
	;			they are always the same.
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(2) set up image type, the read routine takes
	;only specific strings, after the data is read, 
	;the mytype will be used to locate high  
	; m-TOF resolution images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	imagetypes = ['Spatial','Time','m-TOF']	
	mytype = 2
		
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;>>(3) set up units
	;0 = Counts
	;1 = Counts/s
	;2 = Counts/(cm*s*sr)
	;3 = Counts/(cm*s*sr*Kev)
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	unittypes = ['Counts','Counts/s', 'Counts/(cm*s*sr)','Counts/(cm*s*sr*Kev)']
	myunit = 3
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(4) Species, after the data is read, the myspecies variable
	; will be used to locate H images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	species = ['H','NotUsed','O','NotUsed','NotUsed','All']
	myspecies = 0
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(5) if using high m-tof images, the tof = 3 
	; selects the energy range is 55-90 keV  , 
	; after the data is read, the myspecies variable
	; will be used to locate 55-90 keV images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	mytof = 3

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(6) set up titles for charge, after the data is read, the mycharge variable
	; will be used to locate Ion images in the image array
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	charge = ['Neutral','Ion']
	mycharge = 0
		
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(7) set up INCA image and INCA accumulator rates filenames
	;INCA calibration uses the accumulator rates data
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	files = [topdir+'INCA_IMAGES'+p+'2010'+p+'IIMG0_2010267_0003.CSV']
	arfiles = [topdir+'INCA_ACCUMULATION_RATES'+p+'2010'+p+'IACC0_2010267_0003.CSV']
		
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; >>(8) set up time to match file selected
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	startyear = 2010
	startdoy = 267
	starthours = 18
	startmins = 0
	startsecs = 0
	
	stopyear = 2010
	stopdoy = 267
	stophours = 19
	stopmins = 0
	stopsecs = 0
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;convert times to ascii UTC string to ephemeris time, 
	;et is dblarr[2], 0=starting time, 1=ending time
	;ephemeris time makes it easier to find matching times
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	strtime = NUM2TIMESTR(startyear, startdoy, starthours, startmins, startsecs)
	stptime = NUM2TIMESTR(stopyear, stopdoy, stophours, stopmins, stopsecs)
	s=UTC2ET([strtime,stptime],et)

	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;Read in MIMI INCA Ion/Neutral Time range files
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	status = READ_NEUIONMODES(strtime,stptime, neuionmodes)
	
	
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;call read routine, only the image type selected will be returned, but each image 
	; type contains multiple TOF, charge and species images!
	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	s = PDS_READ_MIMIIMG(startyear, startdoy, starthours, startmins, startsecs, $
                    stopyear, stopdoy, stophours, stopmins, stopsecs,  $
					files, arfiles, imagetypes[mytype], unittypes[myunit], $
                    labels, data, images, datautc, stopdatautc,$
					auxdata, auxlabels, bandwidths, energy_ranges)

	IF s EQ 0 THEN BEGIN
		;help,labels, data, images, datautc, auxdata, auxlabels, bandwidths, energy_ranges
		
		
		minet = data[0,DATA_ET_SECS_C]
		maxet = data[N_ELEMENTS(data[*,0])-1,DATA_ET_SECS_C]
		s=ET2UTC([minet,maxet],utc)
		print,'TEST_READ_MIMI: Files contain time range ',utc
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;Load an array the same size as the images data array with the matching INCA 
		;neutral/ion modes
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		modes = LONARR(N_ELEMENTS(data[*,DATA_ET_SECS_C])+5)
		mymode = 0l
		
		FOR n=0,N_ELEMENTS(neuionmodes.mode)-1 DO BEGIN
			s=UTC2ET(neuionmodes.utc[*,n], ni_ets)
			
			mymode = neuionmodes.mode[n]
			at = WHERE(data[*,DATA_ET_SECS_C] GE ni_ets[0] AND $
				data[*,DATA_ET_SECS_C] LE ni_ets[1])
			
			IF at[0] NE -1 THEN BEGIN
				modes[at] = neuionmodes.mode[n]
			ENDIF
		ENDFOR

		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		; search for image time, type, tof and species. Only images with 
		; type = high m-TOF will be returned but the test is left for completness.
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		at = WHERE((data[*,DATA_ET_SECS_C] GE et[0] AND $
				data[*,DATA_ET_SECS_C] LE et[1]) AND $
				(data[*,TYPE_C] EQ mytype AND $			
				data[*,SPECIE_C] EQ myspecies AND $		
				modes[*] EQ mycharge AND $		
				data[*,TOF_C] EQ mytof))
				
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
		;test if a match is found
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;				
		IF at[0] NE -1 THEN BEGIN

			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
			;Loop through images, since only one image type, tof, species and charge were
			;selected, the resultant images should be in ascending time order. However,
			;if more than one tof or species are present in the array, then they will not 
			;be in ascending time order since all tofs, and specie images for an image type
			;are collected at the same time periods.
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			FOR n=0,N_ELEMENTS(datautc[at])-1 DO BEGIN
				isize = LONG(data[at[n],NUMROWS_C])
				strisize = NUM2STR(isize)
				er = energy_ranges[*,at[n]]
				bw = bandwidths[at[n]]
				crg = data[at[n],CHARGE_C]
				utc = STRMID(datautc[at[n]],1,STRLEN(datautc[at[n]])-2)
				
				;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				;save last image to display, save image attributes to write in window
				;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				myimage = REFORM(images[at[n],0:isize-1,0:isize-1])

				print,utc
				print,'energy range ',er,' bandwidth ',bw
				print,charge[data[at[n],CHARGE_C]]+' image'
				FOR i=0,isize-1 DO BEGIN
					PRINT,myimage[i,0:isize-1],$
						format='('+strisize+'(D5.2,","))'
				ENDFOR
			ENDFOR
			
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    		;open a window or the Z buffer used to make 
			; a PNG file
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			IF KEYWORD_SET(MAKE_PNG) THEN BEGIN
        		set_plot,'Z'
        		Device,SET_RESOLUTION=[900,800]
			ENDIF ELSE BEGIN
				WINDOW,/FREE,retain=2,xsize=900,ysize=800
    			WSET,!D.WINDOW
			ENDELSE
			
			;load colormap, put set of known colors on the top 
			;of the colormap for line plots
			colormapsize = 256
			PDS_LOAD_COLORMAP, 33, colormapsize
    
			; setup colors and linestyles 
    		PDS_LOAD_COLOR_INDEXES, 10, colors, lstyles


			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;increase imagesize on window according to 
			;imagesize
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			IF isize EQ 16 THEN BEGIN
				sizetoincrease = 27
			ENDIF ELSE IF isize EQ 32 THEN BEGIN
				sizetoincrease = 15
			ENDIF ELSE BEGIN	
				sizetoincrease = 7
			ENDELSE
			
			;size the image to maintain aspect ratio
			;since there are 90 pixels in azimuth (x axis), 
			;120 pixels in elevation (y axis)
			xsize = isize*sizetoincrease
			ysize = isize*sizetoincrease*(1.2/.9)
			myimage = CONGRID(myimage, xsize, ysize)

			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;Set image color scale min and max or use 
			;	image min and max 
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;xmin = MIN(myimage, MAX=xmax,/NAN)
			xmin = 0.1	
			xmax = 4.0
			
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;display in log
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
        	enamin = ALOG10(xmin > 1e-5)
        	enamax = ALOG10(xmax > 1e-5)
        	myimage = ALOG10(myimage+0.000001)



			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;bytescale to use colormap to display
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    		temp = BYTSCL(myimage,MIN =enamin, MAX=enamax, TOP=colormapsize)
    		TV,temp,0.1,0.1,/normal

			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;plot axis and titles
			;figure size of image in normal coordinates
			;so the axis can be plotted around it
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			temp = CONVERT_COORD(0.1,0.1,/NORM,/TO_DEVICE)
			temp2 = CONVERT_COORD(temp[0]+xsize,temp[1]+ysize,/DEVICE,/TO_NORM)
			pos = [0.1,0.1,temp2[0],temp2[1]]
			PLOT,[-45.,45.],[-60.,60.],/nodata,XSTYLE=17,YSTYLE=17,$
			XTICKS=9, YTICKS=12,XTITLE='Azimuth',YTITLE='Elevation',$
			XRANGE=[-45.,45.],YRANGE=[-60.,60.],POSITION=pos,/NOERASE,/NORM,color=colors[0]

			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;plot colorbar
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        	position = [0.88, 0.1, 0.91, 0.8]
    		BARFORMAT = '(g10.4)'
    		barlevels = 2
        	vertical = 1
    		COLORBAR, position = position, vertical = vertical, $
    		  min = enamin, max = enamax, f = barformat, title = ytitle,  $
    		  ncolors = colormapsize+1, ps = (!d.name eq 'PS'), color = colors[0],  $
    		  charsize = small_text_size, divisions = barlevels > 2


			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			;write some titles
			;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			XYOUTS,0.05,0.95,'Cassini/MIMI INCA High '+imagetypes[mytype]+' Resolution '+$
				species[myspecies]+' Image  '+utc,/normal,charsize=1.5,color=colors[0]
			XYOUTS,0.05,0.9,STRJOIN(STRCOMPRESS(er,/REMOVE_ALL),' - ')+' keV  '+$
				charge[crg]+' image   '+unittypes[myunit],/normal,charsize=1.3,color=colors[0]
			
			IF KEYWORD_SET(MAKE_PNG) THEN BEGIN	
				pngfile = getenv('HOME')+'/test_read_mimiimg.png'
        		pict = TVRD()
        		TVLCT, r,g,b,/get
				PRINT,'Creating PNG '+pngfile
				WRITE_PNG, pngfile, pict, r,g,b
            	device,/close
            	set_plot,'X'
			ENDIF 
			
		ENDIF ELSE BEGIN
			PRINT,'TEST_READ_MIMIIMG: Failure to locate matching image in image array'
		ENDELSE
	ENDIF ELSE BEGIN
			PRINT,'TEST_READ_MIMIIMG: Failure to locate matching data'
	ENDELSE
	
     
RETURN
END
