;+
; NAME:
;	UTC2ET
;
; PURPOSE:
;	Convert from UTC string format to ephemeris time in seconds (TDB). Only use this routine
;   with MIMI data, it may not be accurate enough for other instruments!!!!!!
;
;   WARNING: This routine assumes that TDT equals TDB since the MIMI data 
;   is not affected by differences in time of less than 0.1 second. I found that this routine  
;   calculates an ET with a maximum difference of +/- 0.00165701 seconds over one year , 
;   calculated every minute, when compared with a routine using spice. To replace this routine with
;   SPICE software which requires the MIMI team SPICE analysis kernel set see the included routine 
;   SPICE_ET2UTC.pro
;
;   See "http://naif.jpl.nasa.gov/pub/naif/toolkit_docs/FORTRAN/index.html" Required Reading- Time
;   for more information.
;
; CATEGORY:
;	Plotting Structure File. 
;
; CALLING SEQUENCE:
;	s = UTC2ET(utc, et)
;
; INPUTS:
;  utc  		input string  UTC starttime, yyyy-doyThh:mm.ss.msc (Coordinated Universal Time)
;
; OUTPUTS:
;	et           output array of double to ephermeris time Barycentric dynamical time (TDB), 
;                       seconds from Jan 1, 2000 12:00 UTC
;
; REQUIRES:
;   Uses the leap second common block, assumes setup was run before use
;
;   The leap second file is obtained from "ftp://naif.jpl.nasa.gov/pub/naif/CASSINI/kernels/lsk/" 
;   and the user should check in January and August of each year to see if any leap seconds have 
;   been added.
;
;   RETURNS 0 RETURN will always be 0, left in to make it easier to replace
;		with spice routines.
;          
;
; KEYWORD PARAMETERS:
;
;                       Copyright (C) 2011
; The Johns Hopkins University Applied Physics Laboratory (JHU/APL)
;                       All rights reserved
;
; This material may be used, modified or reproduced by or for the
; U.S. Government pursuant to the license rights granted under
; the clauses at DFARS 252.227-7013/7014.
;
; CREATION:
;       01 Oct 2010, M.B. Kusterer 
;
; MODIFICATION HISTORY: 
;       11 May 2011, M.B. Kusterer Fixed to be more accurate
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


PRO TESTUTC2ET

;this test routine requires SPICE library to be installed and the SPICE DLM to
;   be loaded

    utcs = ['1999-001T12:00:00',$
	    '2000-001T00:00:00',$
	    '2000-001T11:58:55',$
	    '2005-365T23:59:59.999',$
	    '2006-001T00:00:00.000',$
        '2006-002T00:00:00.000',$
	    '2008-366T23:59:59.999',$
	    '2009-001T00:00:00.000',$
        '2009-002T00:00:00.000',$
	    '2011-200T12:00:00']
       
    diff = dblarr(n_elements(utcs))
        
	    FOR n=0,n_elements(utcs)-1 DO BEGIN
		    s=UTC2ET(utcs[n], secs)
		    s=spice_utc2et(utcs[n],et)
            diff[n] = secs[0]-et[0]
            print,utcs[n],secs[0],et[0],diff[n],format='(A23,3(f23.8))'
            print,' '
	    ENDFOR
    print,'difference '
    print,min(diff),max(diff),format='(2(f23.8))'

RETURN
END


FUNCTION UTC2ET, utc, et


COMMON LEAP, ls
   
    ;check that  leap second kernel has been read in and stored in common block
    IF  SIZE(ls,/TYPE) NE 8  THEN BEGIN
        SETUP
        IF  SIZE(ls,/TYPE) NE 8  THEN BEGIN
            PRINT,'READ_LSK: Error, please set LEAPKERNEL_FILE envirionmental variable in setup.pro and run setup'
            RETURN,-1
        ENDIF
    ENDIF

    ;0 ET = 2000-001T11:58:55.816
	TIMESTR2NUM, utc, year, doy, hour, min, sec, msec
    
	num = N_ELEMENTS(utc)
	et = DBLARR(num)
	
	FOR n=0,num-1 DO BEGIN
        ;convert utc to TAI
        tai =  UTC2TAI(utc[n]) 
        
        ;convert TAI to ET
        et[n] = tai[0] + ls.deltaTa[0]  
        
	ENDFOR

RETURN,0
END

